{ **************************************************************************** }
{ *                                                                          * }
{ * DONKEY KONG JR. (Nintendo, Game & Watch Table Top) Simulator             * }
{ * by Luca "MADrigal" Antignano, May 2001 (revision: September 2022)        * }
{ *                                                                          * }
{ * http://www.madrigaldesign.it/sim/                                        * }
{ * lucantignano@gmail.com                                                   * }
{ *                                                                          * }
{ * This source code (the "Software") is released as "open source" under the * }
{ * "Creative Common Attribution-NonCommercial-ShareAlike" license 4.0       * }
{ * available at:                                                            * }
{ * https://creativecommons.org/licenses/by-nc-sa/4.0/legalcode              * }
{ *                                                                          * }
{ * In addition to the constraints set by the CC license, you must respect   * }
{ * the following rules:                                                     * }
{ * a. do not offer the Software on a pay-per-play basis.                    * }
{ * b. do not distribute the Software through pre-paid access websites or    * }
{ *    BBS's.                                                                * }
{ * c. do not use the Software in publications of any form unless written    * }
{ *    consent is obtained by the author.                                    * }
{ * d. do not distribute the Software through mirror websites, unless        * }
{ *    written consent is obtained by the author.                            * }
{ * e. do not distribute older versions of the Software through authorized   * }
{ *    mirror websites after newer ones have been made available by the      * }
{ *    author.                                                               * }
{ *                                                                          * }
{ * Excerpt from the CC BY-NC-SA license 4.0                                 * }
{ * ========================================                                 * }
{ * BY: You must give appropriate credit, provide a link to the license, and * }
{ *     indicate if changes were made. You may do so in any reasonable       * }
{ *     manner, but not in any way that suggests the licensor endorses you   * }
{ *     or your use.                                                         * }
{ * NC: You may not use the material for commercial purposes.                * }
{ * SA: If you remix, transform, or build upon the material, you must        * }
{ *     distribute your contributions under the same license as the          * }
{ *     original.                                                            * }
{ *                                                                          * }
{ **************************************************************************** }

unit unit1;
{ this unit contains the main form implementation                              }

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, Math, Registry, Jpeg, Pngimage, Fmod, Fmodtypes, StdCtrls, Menus,
  ShellAPI;

type
  TForm1 = class(TForm)
{ * HANDHELD GRAPHICS, INCLUDING BUTTONS                                     * }

    // fullscreen mode items
    btn_game_menu_top:          TImage;
    btn_game_menu_background:   TImage;

    im_backdrop:                TImage;
    im_backdrop_logo:           TImage;

    // electronic game
    im_background:              TImage;     im_background_screen_top: TImage;

    btn_joystick_bottom:        TImage;     btn_joystick_top:         TImage;
    btn_joystick_bottom_up:     TImage;
    btn_joystick_bottom_down:   TImage;
    btn_joystick_bottom_left:   TImage;
    btn_joystick_bottom_right:  TImage;

    btn_jump_down:              TImage;     btn_jump_top:             TImage;

    btn_game_a_down:            TImage;     btn_game_a_top:           TImage;
    btn_game_b_down:            TImage;     btn_game_b_top:           TImage;
    btn_time_down:              TImage;     btn_time_top:             TImage;

    btn_acl_down:               TImage;     btn_acl_top:              TImage;
                                            btn_alarm_top:            TImage;

    btn_ctrlr_overlay_up:       TImage;
    btn_ctrlr_overlay_left:     TImage;
    btn_ctrlr_overlay_right:    TImage;
    btn_ctrlr_overlay_down:     TImage;

{ * LCD SCREEN GRAPHICS (SPRITES)                                            * }

    // on-screen panel (background, alarm, time/score, game a/b, etc.)
    im_game_background: TImage;

    im_panel_miss_0: TImage; im_panel_miss_1: TImage; im_panel_miss_2: TImage; im_panel_miss_3: TImage;

    im_panel_alarm_1: TImage; im_panel_alarm_2: TImage; im_panel_alarm_3: TImage;

    im_panel_game_a: TImage; im_panel_game_b: TImage;

    im_panel_time_am: TImage; im_panel_time_pm: TImage; im_panel_time_colon: TImage;

    im_panel_digit_1: TImage; im_panel_digit_2: TImage; im_panel_digit_3: TImage; im_panel_digit_4: TImage;
    im_panel_number_0: TImage; im_panel_number_1: TImage; im_panel_number_2: TImage; im_panel_number_3: TImage; im_panel_number_4: TImage;
    im_panel_number_5: TImage; im_panel_number_6: TImage; im_panel_number_7: TImage; im_panel_number_8: TImage; im_panel_number_9: TImage;

    // on-screen items
    im_junior_1_1_hand_1: TImage;
    im_junior_1_1_hand_2: TImage;
    im_junior_1_1: TImage; im_junior_2_1: TImage; im_junior_3_1: TImage; im_junior_4_1: TImage; im_junior_5_1: TImage; im_junior_6_1: TImage;                             im_junior_victory: TImage;
    im_junior_1_2: TImage; im_junior_2_2: TImage; im_junior_3_2: TImage;                        im_junior_5_2: TImage; im_junior_6_2: TImage; im_junior_6_2_hand: TImage;
    im_junior_1_3: TImage; im_junior_2_3: TImage; im_junior_3_3: TImage; im_junior_4_3: TImage; im_junior_5_3: TImage; im_junior_6_3: TImage; im_junior_6_3_hand: TImage;
    im_junior_1_4: TImage; im_junior_2_4: TImage; im_junior_3_4: TImage; im_junior_4_4: TImage; im_junior_5_4: TImage; im_junior_6_4: TImage; im_junior_6_4_hand: TImage;
    im_junior_1_5: TImage; im_junior_2_5: TImage; im_junior_3_5: TImage;

                                                                   im_crow_4_0: TImage; im_crow_5_0: TImage;
    im_crow_1_1: TImage; im_crow_2_1: TImage; im_crow_3_1: TImage; im_crow_4_1: TImage; im_crow_5_1: TImage;
    im_crow_1_2: TImage; im_crow_2_2: TImage; im_crow_3_2: TImage; im_crow_4_2: TImage; im_crow_5_2: TImage;
    im_crow_1_3: TImage; im_crow_2_3: TImage; im_crow_3_3: TImage; im_crow_4_3: TImage; im_crow_5_3: TImage;
    im_crow_1_4: TImage; im_crow_2_4: TImage; im_crow_3_4: TImage; im_crow_4_4: TImage;
    im_crow_1_5: TImage; im_crow_2_5: TImage; im_crow_3_5: TImage; im_crow_4_5: TImage; im_crow_5_5: TImage;

    im_key_1: TImage; im_key_2: TImage;

    im_coconut_0: TImage;                        im_balloon_0: TImage;
    im_coconut_1: TImage; im_umbrella_1: TImage; im_balloon_1: TImage;
    im_coconut_2: TImage; im_umbrella_2: TImage; im_balloon_2: TImage;
    im_coconut_3: TImage; im_umbrella_3: TImage; im_balloon_3: TImage;
    im_coconut_4: TImage; im_umbrella_4: TImage; im_balloon_4: TImage;
    im_coconut_5: TImage; im_umbrella_5: TImage; im_balloon_5: TImage;

    im_kong_body: TImage;
    im_kong_arm_l2: TImage; im_kong_arm_l3: TImage; im_kong_arm_l4: TImage;
    im_kong_arm_r1: TImage; im_kong_arm_r2: TImage;
    im_kong_lock_l2: TImage; im_kong_lock_l3: TImage;im_kong_lock_l4: TImage;
    im_kong_chain_1: TImage; im_kong_chain_2: TImage; im_kong_chain_3: TImage; im_kong_chain_4: TImage;

    im_miss_1: TImage; im_miss_2: TImage;

{ * TIMERS                                                                   * }
    Timer_Bonus_Phase_1:    TTimer;
    Timer_Bonus_Phase_2:    TTimer;
    Timer_Chance:           TTimer;
    Timer_Clock:            TTimer;
    Timer_Coconut:          TTimer;
    Timer_Coconut_Bonus:    TTimer;
    Timer_Coconut_Music:    TTimer;
    Timer_Flying_Music:     TTimer;
    Timer_Game:             TTimer;
    Timer_Game_Over:        TTimer;
    Timer_Game_Start:       TTimer;
    Timer_Junior_1_1_Hand:  TTimer;
    Timer_Junior_6_Hand:    TTimer;
    Timer_Junior_Jump:      TTimer;
    Timer_Junior_Step:      TTimer;
    Timer_Key_Blink:        TTimer;
    Timer_Key_Fall:         TTimer;
    Timer_Kong_Arm:         TTimer;
    Timer_Miss_Crow:        TTimer;
    Timer_Miss_Fall:        TTimer;
    Timer_Score_Blink:      TTimer;
    Timer_Umbrella_Balloon: TTimer;

{ * POPUP MENU                                                               * }
    pm_0_menu:          TPopupMenu;
    pm_1_about:         TMenuItem;
    pm_2_readme:        TMenuItem;
    pm_3_whatsnew:      TMenuItem;
    pm_4_manual:        TMenuItem;
    pm_5_n1:            TMenuItem;
    pm_6_config:        TMenuItem;
    pm_7_keys:          TMenuItem;
    pm_8_hiscore:       TMenuItem;
    pm_9_n2:            TMenuItem;
  {}pm_10_fullscreen:   TMenuItem;
  {}pm_11_splashscreen: TMenuItem;
  {}pm_12_controller:   TMenuItem;
  {}pm_13_n3:           TMenuItem;
  {}pm_14_exit:         TMenuItem;

{ * PROCEDURES: INITIALIZE & SHUT DOWN                                       * }
    procedure FormCreate                    (Sender: TObject);
    procedure FormActivate                  (Sender: TObject);
    procedure FormClose                     (Sender: TObject; var Action: TCloseAction);

{ * PROCEDURES: DISPLAY MANAGEMENT                                           * }
    procedure Vid_Display_Ctrl_Overlay;

    procedure Vid_Show_All                  (b: boolean);

    procedure Vid_Show_All_Hero             (b: boolean);
    procedure Vid_Show_All_Crows            (b: boolean);
    procedure Vid_Show_All_Coconuts         (b: boolean);
    procedure Vid_Show_All_Umbrellas        (b: boolean);
    procedure Vid_Show_All_Balloons         (b: boolean);
    procedure Vid_Show_All_Kong_Body        (b: boolean);
    procedure Vid_Show_All_Kong_Chains      (b: boolean);
    procedure Vid_Show_All_Keys             (b: boolean);
    procedure Vid_Show_All_Miss             (b: boolean);

    procedure Vid_Show_Panel_Score          (var p: integer; b: boolean);
    procedure Vid_Show_Panel_Miss           (var p: integer);

    procedure Vid_Show_Hero                 (b: boolean);
    procedure Vid_Show_Coconut              (b: boolean);
    procedure Vid_Show_Kong_Arm_Lock        (b: boolean);

{ * PROCEDURES: AUDIO MANAGEMENT                                             * }
    procedure Snd_Play                      (pfs: PFSoundSample);
    procedure Snd_Silence;

{ * PROCEDURES: INPUT MANAGEMENT - EVENTS                                    * }
    procedure im_backgroundMouseDown        (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure im_backgroundMouseMove        (Sender: TObject;                       Shift: TShiftState; X, Y: Integer);

    procedure btn_game_a_topMouseDown       (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_game_a_topMouseUp         (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_game_b_topMouseDown       (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_game_b_topMouseUp         (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_time_topMouseDown         (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_time_topMouseUp           (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_acl_topMouseDown          (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_acl_topMouseUp            (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);

    procedure btn_jump_topMouseDown         (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_jump_topMouseUp           (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);

    procedure btn_ctrlr_overlay_upMouseDown (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_ctrlr_overlay_upMouseUp   (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_ctrlr_overlay_rightMouseDown(Sender:TObject;Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_ctrlr_overlay_rightMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_ctrlr_overlay_leftMouseDown(Sender:TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_ctrlr_overlay_leftMouseUp (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_ctrlr_overlay_downMouseDown(Sender:TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure btn_ctrlr_overlay_downMouseUp (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);

    procedure FormKeyDown                   (Sender: TObject; var Key: Word;        Shift: TShiftState);
    procedure FormKeyUp                     (Sender: TObject; var Key: Word;        Shift: TShiftState);

    procedure Inp_game_a_down;              procedure Inp_game_a_up;
    procedure Inp_game_b_down;              procedure Inp_game_b_up;
    procedure Inp_time_down;                procedure Inp_time_up;
    procedure Inp_acl_down;                 procedure Inp_acl_up;

    procedure Inp_up_down;                  procedure Inp_up_up;
    procedure Inp_down_down;                procedure Inp_down_up;
    procedure Inp_left_down;                procedure Inp_left_up;
    procedure Inp_right_down;               procedure Inp_right_up;

    procedure Inp_jump_down;                procedure Inp_jump_up;

{ * PROCEDURES: GAME                                                         * }
    procedure Gam_Stop_All_Timers;

    procedure Gam_Check_Chance_Time;
    function  Gam_Check_Collisions: boolean;
    procedure Gam_Execute_Miss_Crow;
    procedure Gam_Execute_Miss_Fall;
    procedure Gam_Reprise_After_Miss;
    procedure Gam_Set_Difficulty;
    procedure Gam_Set_Mode;

{ * PROCEDURES: TIMERS                                                       * }
    procedure Timer_Bonus_Phase_1Timer      (Sender: TObject);
    procedure Timer_Bonus_Phase_2Timer      (Sender: TObject);
    procedure Timer_ChanceTimer             (Sender: TObject);
    procedure Timer_ClockTimer              (Sender: TObject);
    procedure Timer_CoconutTimer            (Sender: TObject);
    procedure Timer_Coconut_BonusTimer      (Sender: TObject);
    procedure Timer_Coconut_MusicTimer      (Sender: TObject);
    procedure Timer_Flying_MusicTimer       (Sender: TObject);
    procedure Timer_GameTimer               (Sender: TObject);
    procedure Timer_Game_OverTimer          (Sender: TObject);
    procedure Timer_Game_StartTimer         (Sender: TObject);
    procedure Timer_Junior_1_1_HandTimer    (Sender: TObject);
    procedure Timer_Junior_6_HandTimer      (Sender: TObject);
    procedure Timer_Junior_JumpTimer        (Sender: TObject);
    procedure Timer_Junior_StepTimer        (Sender: TObject);
    procedure Timer_Key_BlinkTimer          (Sender: TObject);
    procedure Timer_Key_FallTimer           (Sender: TObject);
    procedure Timer_Kong_ArmTimer           (Sender: TObject);
    procedure Timer_Miss_CrowTimer          (Sender: TObject);
    procedure Timer_Miss_FallTimer          (Sender: TObject);
    procedure Timer_Score_BlinkTimer        (Sender: TObject);
    procedure Timer_Umbrella_BalloonTimer   (Sender: TObject);

{ * PROCEDURES: POPUP MENU MANAGEMENT                                        * }
    procedure pm_0_menuPopup                (Sender: TObject);
    procedure pm_1_aboutClick               (Sender: TObject);
    procedure pm_2_readmeClick              (Sender: TObject);
    procedure pm_3_whatsnewClick            (Sender: TObject);
    procedure pm_4_manualClick              (Sender: TObject);
    procedure pm_6_configClick              (Sender: TObject);
    procedure pm_7_keysClick                (Sender: TObject);
    procedure pm_8_hiscoreClick             (Sender: TObject);
  {}procedure pm_10_fullscreenClick         (Sender: TObject);
  {}procedure pm_11_splashscreenClick       (Sender: TObject);
  {}procedure pm_12_controllerClick         (Sender: TObject);
  {}procedure pm_14_exitClick               (Sender: TObject);

{ * PROCEDURES: GAME MENU MANAGEMENT                                         * }
    procedure btn_game_menu_topClick        (Sender: TObject);

    procedure la_menu_itemMouseEnter        (Sender: TObject);
    procedure la_menu_itemMouseLeave        (Sender: TObject);
    procedure la_menu_itemMouseUp           (Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);

    procedure Vid_Display_Game_Menu         (b: boolean);
  end;

const
{ introduced for use with HH2 / now incorporated into main source }

  HH2_CONTENT_NAME    = 'Donkey Kong Junior (Nintendo, Game & Watch Table Top)';
  HH2_CONTENT_AUTHOR  = 'Luca "MADrigal" Antignano';
  HH2_CONTENT_EMAIL   = 'lucantignano@gmail.com';
  HH2_CONTENT_VERSION = '5.01';
  HH2_CONTENT_DATE    = '2022/09/12';
  HH2_CONTENT_WEBSITE = 'www.madrigaldesign.it/sim';

  { constants for all builds }
  
  APP_NAME      = HH2_CONTENT_NAME + ' Simulator';                              // application's name (appears on taskbar and Splash screen)
{}APP_VERSION   = 'Version ' + HH2_CONTENT_VERSION + ' * Graphic improvements *'; // current version (appears on Splash screen)
{}APP_DATE      = 'Released by ' + HH2_CONTENT_AUTHOR + ' on 12th September, 2022'; // release date (appears on Splash screen)

  URL_EMAIL     = 'mailto:' + HH2_CONTENT_EMAIL;                                // link to the email address
  URL_PAYPAL    = 'https://www.paypal.me/LucaAntignano';                        // link to the Paypal donation page
  URL_WEBSITE   = 'http://' + HH2_CONTENT_WEBSITE + '/';                        // link to the website

  REG_KEY       = 'Software\MADrigal\DKongJrT50';                               // registry key

  HINT_CLICK    = 'click or press';
  HINT_GAME_A   = 'to start game A (hold to view highscore)';
  HINT_GAME_B   = 'to start game B (hold to view highscore)';
  HINT_TIME     = 'to view current time (hold to view alarm time)';
  HINT_ACL      = 'to reset the game';
  HINT_ALARM    = 'button disabled';
  HINT_JOYSTICK = 'activate the controller overlays or use';

{}MENU_ITEMS = 14;

var
{ * main objects * }
  Form1:                   TForm1;                                              // main form
  bStartup:                boolean;                                             // manages splash screen at startup
  iMouseX, iMouseY:        integer;                                             // cursor coordinates (to drag main form)
  iReg:                    TRegIniFile;                                         // system registry (to save high-scores)

{ * sound management * }
  pfs_bonus1,
  pfs_bonus2,
  pfs_bonus3,
  pfs_chance,
  pfs_coconut,
  pfs_crow,
  pfs_falling,
  pfs_flying,
  pfs_hand1,
  pfs_hand2,
  pfs_jump,
  pfs_key1,
  pfs_key2,
  pfs_miss,
  pfs_point,
  pfs_start1,
  pfs_start2,
  pfs_step:                PFSoundSample;                                       // sound samples

  bSound:                  boolean;                                             // sound enabled? (depending on selected driver on splash screen)

{ * arrays of images * }
  aIm_panel_miss:          array[0..3]       of TImage;                         // miss marks panel
  aIm_panel_alarm:         array[1..3]       of TImage;                         // alarm panel
  aIm_panel_digit_pos:     array[1..4]       of TImage;                         // empty sprites to host score/time digits
  aIm_panel_number:        array[0..9]       of TImage;                         // digits for score/time

  aIm_junior:              array[1..6, 1..5] of TImage;
  aIm_junior_hand_1:       array[1..2]       of TImage;                         // used when Junior in (1,1)
  aIm_junior_hand_6:       array[2..4]       of TImage;                         // used when Junior in (6,x)
  aIm_crow:                array[1..5, 0..5] of TImage;
  aIm_umbrella:            array[1..5]       of TImage;
  aIm_coconut,
  aIm_balloon:             array[0..5]       of TImage;
  aIm_kong_arm,
  aIm_kong_lock:           array[2..4]       of TImage;
  aIm_kong_chain:          array[1..4]       of TImage;
  aIm_key,
  aIm_miss:                array[1..2]       of TImage;

{ * arrays of mixed types *}
  aKeys:                   array[1..9]       of record                          // keys management (values)
                                                  int:  integer;
                                                  word: word;
                                                end;

  aKeyHints:               array[1..44] of string;                              // keys management (hint on buttons)

  aMenuItem:               array[1..MENU_ITEMS] of TMenuItem;                   // popup menu items
  aMenuLabels:             array[1..MENU_ITEMS] of TLabel;                      // on-screen game menu labels

{ * integers and booleans * }
  iMode,                                                                        // 0=closed, 1/2=ACL 1/2, 3=alarm, 4=time, 5/7=hiscore A/B, 6/8=game A/B, 9=game over
  iScore,                                                                       // score=0-999
  iHiScoreA,                                                                    // hiscore for Game A=0-999
  iHiScoreB,                                                                    // hiscore for Game B=0-999
  iScoreMultiplier,                                                             // given points=1 (normal) or 2 (chance-time)
  iScoreBonusValue,                                                             // bonus points given when a chain is removed
  iScoreNextChance,                                                             // 300 or 1300 (then reset score)
  iScoreCoconut,                                                                // bonus points given when a crow is hit by a coconut

  iPosHeroX,                                                                    // X coordinate of hero
  iPosHeroY,                                                                    // Y coordinate of hero
  iMiss,                                                                        // number of misses: 0-3
  iMaxRandom,                                                                   // maximum amount of crows on screen (depending on value of iScore)

  iKeyStatus,                                                                   // 1=blinking; 2=Junior has it
  iChainsRemoved,                                                               // 1-4
  iKongsFreed                                                                   // 1-? (needed to identify which pf_bonusX to play)
                         : integer;

{}bFullscreenMode,                                                              // true if video mode is fullscreen (false if "shaped game")
  bGameMenuOpen,                                                                // true if the on-screen menu is visible
  bDisplayCtrlOverlays,                                                         // true if the controller overlay should be displayed

  bKeypressed,                                                                  // true while any left/right key is pressed
  bControls,                                                                    // true while player can actually move the hero

  bChance                                                                       // true while the game is in "Chance-time"
                         : boolean;

  // Game variables: loops
  iChanceStatus,

  iLoopBonusPhase1,
  iLoopBonusPhase2,
  iLoopChance,
  iLoopCoconut,
  iLoopCoconutBonus,
  iLoopGame,
  iLoopGameOver,
  iLoopGameStart,
  iLoopJunior11Hand,
  iLoopJuniorJump,
  iLoopKeyFall,
  iLoopKongArm,
  iLoopMissCrow,
  iLoopMissFall,
  iLoopScoreBlink,
  iLoopUmbrellaBalloon
                         : integer;

implementation

uses
  about, gfxinit, keys, manual, splash;

{$r *.dfm}










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: INITIALIZE & SHUT DOWN                                       * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.FormCreate(Sender: TObject);
{ sprites on form are associated to arrays of images, needed to "index" them   }
{ variables are initialized: system registry, randomize, game mode, etc.       }
  var
    i: integer;
begin
  // initializes application name (for Windows taskbar)
  Application.Title := APP_NAME;

{}// sets game off the screen while graphics initialise
{}Top  := - Screen.Height;
{}Left := - Screen.Width;

  // on-screen panel
  aIm_panel_miss[0] := im_panel_miss_0; aIm_panel_miss[1] := im_panel_miss_1; aIm_panel_miss[2] := im_panel_miss_2; aIm_panel_miss[3] := im_panel_miss_3;
  aIm_panel_alarm[1] := im_panel_alarm_1; aIm_panel_alarm[2] := im_panel_alarm_2; aIm_panel_alarm[3] := im_panel_alarm_3;
  aIm_panel_digit_pos[1] := im_panel_digit_1; aIm_panel_digit_pos[2] := im_panel_digit_2; aIm_panel_digit_pos[3] := im_panel_digit_3; aIm_panel_digit_pos[4] := im_panel_digit_4;

  // numbers
  aIm_panel_number[0] := im_panel_number_0; aIm_panel_number[1] := im_panel_number_1; aIm_panel_number[2] := im_panel_number_2; aIm_panel_number[3] := im_panel_number_3; aIm_panel_number[4] := im_panel_number_4;
  aIm_panel_number[5] := im_panel_number_5; aIm_panel_number[6] := im_panel_number_6; aIm_panel_number[7] := im_panel_number_7; aIm_panel_number[8] := im_panel_number_8; aIm_panel_number[9] := im_panel_number_9;

  // on-screen items
  aIm_junior_hand_1[1] := im_junior_1_1_hand_1;
  aIm_junior_hand_1[2] := im_junior_1_1_hand_2;
  aIm_junior[1, 1] := im_junior_1_1; aIm_junior[2, 1] := im_junior_2_1; aIm_junior[3, 1] := im_junior_3_1; aIm_junior[4, 1] := im_junior_4_1; aIm_junior[5, 1] := im_junior_5_1; aIm_junior[6, 1] := im_junior_6_1;
  aIm_junior[1, 2] := im_junior_1_2; aIm_junior[2, 2] := im_junior_2_2; aIm_junior[3, 2] := im_junior_3_2;                                    aIm_junior[5, 2] := im_junior_5_2; aIm_junior[6, 2] := im_junior_6_2; aIm_junior_hand_6[2] := im_junior_6_2_hand;
  aIm_junior[1, 3] := im_junior_1_3; aIm_junior[2, 3] := im_junior_2_3; aIm_junior[3, 3] := im_junior_3_3; aIm_junior[4, 3] := im_junior_4_3; aIm_junior[5, 3] := im_junior_5_3; aIm_junior[6, 3] := im_junior_6_3; aIm_junior_hand_6[3] := im_junior_6_3_hand;
  aIm_junior[1, 4] := im_junior_1_4; aIm_junior[2, 4] := im_junior_2_4; aIm_junior[3, 4] := im_junior_3_4; aIm_junior[4, 4] := im_junior_4_4; aIm_junior[5, 4] := im_junior_5_4; aIm_junior[6, 4] := im_junior_6_4; aIm_junior_hand_6[4] := im_junior_6_4_hand;
  aIm_junior[1, 5] := im_junior_1_5; aIm_junior[2, 5] := im_junior_2_5; aIm_junior[3, 5] := im_junior_3_5;

                                                                                                           aIm_crow[4, 0] := im_crow_4_0;     aIm_crow[5, 0] := im_crow_5_0;
  aIm_crow[1, 1] := im_crow_1_1;     aIm_crow[2, 1] := im_crow_2_1;     aIm_crow[3, 1] := im_crow_3_1;     aIm_crow[4, 1] := im_crow_4_1;     aIm_crow[5, 1] := im_crow_5_1;
  aIm_crow[1, 2] := im_crow_1_2;     aIm_crow[2, 2] := im_crow_2_2;     aIm_crow[3, 2] := im_crow_3_2;     aIm_crow[4, 2] := im_crow_4_2;     aIm_crow[5, 2] := im_crow_5_2;
  aIm_crow[1, 3] := im_crow_1_3;     aIm_crow[2, 3] := im_crow_2_3;     aIm_crow[3, 3] := im_crow_3_3;     aIm_crow[4, 3] := im_crow_4_3;     aIm_crow[5, 3] := im_crow_5_3;
  aIm_crow[1, 4] := im_crow_1_4;     aIm_crow[2, 4] := im_crow_2_4;     aIm_crow[3, 4] := im_crow_3_4;     aIm_crow[4, 4] := im_crow_4_4;
  aIm_crow[1, 5] := im_crow_1_5;     aIm_crow[2, 5] := im_crow_2_5;     aIm_crow[3, 5] := im_crow_3_5;     aIm_crow[4, 5] := im_crow_4_5;     aIm_crow[5, 5] := im_crow_5_5;

  aIm_coconut[0] := im_coconut_0; aIm_coconut[1] := im_coconut_1; aIm_coconut[2] := im_coconut_2; aIm_coconut[3] := im_coconut_3; aIm_coconut[4] := im_coconut_4; aIm_coconut[5] := im_coconut_5;

                                    aIm_balloon[0] := im_balloon_0;
  aIm_umbrella[1] := im_umbrella_1; aIm_balloon[1] := im_balloon_1;
  aIm_umbrella[2] := im_umbrella_2; aIm_balloon[2] := im_balloon_2;
  aIm_umbrella[3] := im_umbrella_3; aIm_balloon[3] := im_balloon_3;
  aIm_umbrella[4] := im_umbrella_4; aIm_balloon[4] := im_balloon_4;
  aIm_umbrella[5] := im_umbrella_5; aIm_balloon[5] := im_balloon_5;

  aIm_kong_lock[2] :=im_kong_lock_l2; aIm_kong_arm[2] := im_kong_arm_l2;
  aIm_kong_lock[3] :=im_kong_lock_l3; aIm_kong_arm[3] := im_kong_arm_l3;
  aIm_kong_lock[4] :=im_kong_lock_l4; aIm_kong_arm[4] := im_kong_arm_l4;

  aIm_kong_chain[1] := im_kong_chain_1; aIm_kong_chain[2] := im_kong_chain_2; aIm_kong_chain[3] := im_kong_chain_3; aIm_kong_chain[4] := im_kong_chain_4;

  aIm_key[1] := im_key_1; aIm_key[2] := im_key_2;

  aIm_miss[1] := im_miss_1; aIm_miss[2] := im_miss_2;

  // registry keys management
  iReg := TregIniFile.Create(REG_KEY);                                          // system registry init

{}// init graphics
{}bFullscreenMode := iReg.ReadBool('', 'FullscreenMode', true);
{}pm_10_fullscreen.Checked := bFullscreenMode;
{}Gfxinit.Gfx_Game_Init;
{}bGameMenuOpen := false;

{}// initialises the "hints" array
  for i := 1 to 10 do
    aKeyHints[i] := '''' + Chr(i + 47) + '''';

  for i := 11 to 36 do
    aKeyHints[i] := '''' + Chr(i + 54) + '''';

  aKeyHints[37] := '''up arrow'' key';
  aKeyHints[38] := '''down arrow'' key';
  aKeyHints[39] := '''left arrow'' key';
  aKeyHints[40] := '''right arrow'' key';
  aKeyHints[41] := '''alt'' key';
  aKeyHints[42] := '''ctrl'' key';
  aKeyHints[43] := '''shift'' key';
  aKeyHints[44] := '''spacebar''';

  // reads values from system registry into the inputs array
  aKeys[1].int := iReg.ReadInteger('', 'Key1', 2);                              // "Game A" (default: "1")
  aKeys[2].int := iReg.ReadInteger('', 'Key2', 3);                              // "Game B" (default: "2")
  aKeys[3].int := iReg.ReadInteger('', 'Key3', 4);                              // "Time"   (default: "3")
  aKeys[4].int := iReg.ReadInteger('', 'Key4', 6);                              // "ACL"    (default: "5")
  aKeys[5].int := iReg.ReadInteger('', 'Key5', 37);                             // "Up"     (default: "UP")
  aKeys[6].int := iReg.ReadInteger('', 'Key6', 39);                             // "Left"   (default: "LEFT")
  aKeys[7].int := iReg.ReadInteger('', 'Key7', 40);                             // "Right"  (default: "RIGHT")
  aKeys[8].int := iReg.ReadInteger('', 'Key8', 38);                             // "Down"   (default: "DOWN")
  aKeys[9].int := iReg.ReadInteger('', 'Key9', 42);                             // "Jump"   (default: "CTRL")

  // writes values into the system registry (in case no pre-existing values
  // stored), then assigns key values to the key-inputs array
  for i := 1 to 9 do
    begin
      iReg.WriteInteger('', 'Key' + inttostr(i), aKeys[i].int);
      KeysForm.AssignWord(i, aKeys[i].int);
    end;

  // sets button images hints
  btn_game_a_top.Hint          := HINT_CLICK + ' ' + aKeyHints[aKeys[1].int] + ' ' + HINT_GAME_A; // "Game A"
  btn_game_b_top.Hint          := HINT_CLICK + ' ' + aKeyHints[aKeys[2].int] + ' ' + HINT_GAME_B; // "Game B"
  btn_time_top.Hint            := HINT_CLICK + ' ' + aKeyHints[aKeys[3].int] + ' ' + HINT_TIME;   // "Time"
  btn_acl_top.Hint             := HINT_CLICK + ' ' + aKeyHints[aKeys[4].int] + ' ' + HINT_ACL;    // "ACL"
  btn_alarm_top.Hint           := HINT_ALARM;                                                     // "Alarm"

  btn_joystick_top.Hint        := HINT_JOYSTICK + ' ' + aKeyHints[aKeys[5].int] + ', '            // "Up"
                                                      + aKeyHints[aKeys[6].int] + ', '            // "Left"
                                                      + aKeyHints[aKeys[7].int] + ', '            // "Right"
                                                      + aKeyHints[aKeys[8].int];                  // "Down"

  btn_jump_top.Hint            := HINT_CLICK + ' ' + aKeyHints[aKeys[9].int];                     // "Jump"

  btn_ctrlr_overlay_up.Hint    := HINT_CLICK + ' ' + aKeyHints[aKeys[5].int];                     // "Up"
  btn_ctrlr_overlay_left.Hint  := HINT_CLICK + ' ' + aKeyHints[aKeys[6].int];                     // "Left"
  btn_ctrlr_overlay_right.Hint := HINT_CLICK + ' ' + aKeyHints[aKeys[7].int];                     // "Right"
  btn_ctrlr_overlay_down.Hint  := HINT_CLICK + ' ' + aKeyHints[aKeys[8].int];                     // "Down"

  // popup menu array association
  aMenuItem[1]  := pm_1_about;
  aMenuItem[2]  := pm_2_readme;
  aMenuItem[3]  := pm_3_whatsnew;
  aMenuItem[4]  := pm_4_manual;
  aMenuItem[5]  := pm_5_n1;
  aMenuItem[6]  := pm_6_config;
  aMenuItem[7]  := pm_7_keys;
  aMenuItem[8]  := pm_8_hiscore;
  aMenuItem[9]  := pm_9_n2;
{}aMenuItem[10] := pm_10_fullscreen;
{}aMenuItem[11] := pm_11_splashscreen;
{}aMenuItem[12] := pm_12_controller;
{}aMenuItem[13] := pm_13_n3;
{}aMenuItem[14] := pm_14_exit;

  // controller overlay setting
{}pm_12_controller.Checked  := iReg.ReadBool('', 'CtrlrOverlay', true);
{}bDisplayCtrlOverlays := pm_12_controller.Checked;

  // controller overlay
  Vid_Display_Ctrl_Overlay;

  // variables init
  DoubleBuffered := true;                                                       // enables double buffering for rendering of "form1"

  iHiScoreA := iReg.ReadInteger('', 'HiScore1', iHiScoreA);                     // reads value from system registry into iHiScoreA
  iHiScoreB := iReg.ReadInteger('', 'HiScore2', iHiScoreB);                     // reads value from system registry into iHiScoreB

  bControls := false;                                                           // prevents hero movement
  bKeyPressed := false;                                                         // no key is currently being kept pressed

  Randomize;                                                                    // initializes the random number generator
end;





procedure TForm1.FormActivate(Sender: TObject);
{ sends focus to the splash screen at startup                                  }
begin
  if bStartup then
    begin
      bStartup := false;
      SplashForm.Show;
    end;
end;





procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
{ frees memory before shutting down                                            }
begin
  FSOUND_Sample_Free(pfs_bonus1);  FSOUND_Sample_Free(pfs_bonus2);
  FSOUND_Sample_Free(pfs_bonus3);  FSOUND_Sample_Free(pfs_chance);
  FSOUND_Sample_Free(pfs_coconut); FSOUND_Sample_Free(pfs_crow);
  FSOUND_Sample_Free(pfs_falling); FSOUND_Sample_Free(pfs_flying);
  FSOUND_Sample_Free(pfs_hand1);   FSOUND_Sample_Free(pfs_hand2);
  FSOUND_Sample_Free(pfs_jump);    FSOUND_Sample_Free(pfs_key1);
  FSOUND_Sample_Free(pfs_key2);    FSOUND_Sample_Free(pfs_miss);
  FSOUND_Sample_Free(pfs_point);   FSOUND_Sample_Free(pfs_start1);
  FSOUND_Sample_Free(pfs_start2);  FSOUND_Sample_Free(pfs_step);

  FSOUND_Close;

{}iReg.Destroy;
  Action := caFree;
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: DISPLAY MANAGEMENT                                           * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Vid_Display_Ctrl_Overlay;
{ displays or hides the controller overlay images                              }
begin
  btn_ctrlr_overlay_up.Visible    := bDisplayCtrlOverlays;
  btn_ctrlr_overlay_left.Visible  := bDisplayCtrlOverlays;
  btn_ctrlr_overlay_right.Visible := bDisplayCtrlOverlays;
  btn_ctrlr_overlay_down.Visible  := bDisplayCtrlOverlays;
end;





procedure TForm1.Vid_Show_All(b: boolean);
{ displays/hides all sprites                                                   }
  var
    i: integer;
begin
  for i := 1 to 4 do
    aIm_panel_digit_pos[i].Visible := true;

  if b then
    begin
      aIm_panel_digit_pos[1].Picture := aIm_panel_number[1].Picture;
      aIm_panel_digit_pos[2].Picture := aIm_panel_number[2].Picture;
      aIm_panel_digit_pos[3].Picture := aIm_panel_number[0].Picture;
      aIm_panel_digit_pos[4].Picture := aIm_panel_number[0].Picture;
    end
  else
    for i := 1 to 4 do
      aIm_panel_digit_pos[i].Picture := nil;

  i := Ord(b) * 3; Vid_Show_Panel_Miss(i);

  for i := 1 to 3 do
    aIm_panel_alarm[i].Visible := b;

  im_game_background.Visible := b;

  im_panel_time_am.Visible := b;
  im_panel_time_pm.Visible := false;
  im_panel_time_colon.Visible := b;

  im_panel_game_a.Visible := b;
  im_panel_game_b.Visible := b;

  Vid_Show_All_Hero(b);
  Vid_Show_All_Crows(b);
  Vid_Show_All_Coconuts(b);
  Vid_Show_All_Umbrellas(b);
  Vid_Show_All_Balloons(b);
  Vid_Show_All_Kong_Body(b);
  Vid_Show_All_Kong_Chains(b);
  Vid_Show_All_Keys(b);
  Vid_Show_All_Miss(b);
end;





procedure TForm1.Vid_Show_All_Hero(b: boolean);
{ displays/hides all hero's sprites at once                                    }
  var
    i, j: integer;
begin
  for i := 1 to 2 do
    aIm_junior_hand_1[i].Visible := b;

  for i := 1 to 6 do
    aIm_junior[i, 1].Visible := b;

  for i := 1 to 3 do
    begin
      aIm_junior[i, 2].Visible := b;
      aIm_junior[i, 5].Visible := b;
    end;

  for i := 5 to 6 do
    aIm_junior[i, 2].Visible := b;

  for j := 3 to 4 do
    for i := 1 to 6 do
      aIm_junior[i, j].Visible := b;

  for i := 2 to 4 do
    aIm_junior_hand_6[i].Visible := b;

  im_junior_victory.Visible := b;
end;





procedure TForm1.Vid_Show_All_Crows(b: boolean);
{ displays/hides all crows at once                                             }
  var
    i, j: integer;
begin
  for i := 4 to 5 do
    aIm_crow[i, 0].Visible := b;

  for j := 1 to 3 do
    for i := 1 to 5 do
      aIm_crow[i, j].Visible := b;

  for i := 1 to 4 do
    aIm_crow[i, 4].Visible := b;

  for i := 1 to 5 do
    aIm_crow[i, 5].Visible := b;
end;





procedure TForm1.Vid_Show_All_Coconuts(b: boolean);
{ displays/hides all coconuts at once                                          }
  var
    i: integer;
begin
  for i := 0 to 5 do
    aIm_coconut[i].Visible := b;
end;





procedure TForm1.Vid_Show_All_Umbrellas(b: boolean);
{ displays/hides all umbrellas at once                                         }
  var
    i: integer;
begin
  for i := 1 to 5 do
    aIm_umbrella[i].Visible := b;
end;





procedure TForm1.Vid_Show_All_Balloons(b: boolean);
{ displays/hides all balloons at once                                          }
  var
    i: integer;
begin
  for i := 0 to 5 do
    aIm_balloon[i].Visible := b;
end;





procedure TForm1.Vid_Show_All_Kong_Body(b: boolean);
{ displays/hides all Donkey Kong's sprites - except chains                     }
  var
    i: integer;
begin
  for i := 2 to 4 do
    begin
      aIm_kong_arm[i].Visible := b;
      aIm_kong_lock[i].Visible := b;
    end;

  im_kong_body.Visible := b;

  im_kong_arm_r1.Visible := b;
  im_kong_arm_r2.Visible := b;
end;





procedure TForm1.Vid_Show_All_Kong_Chains(b: boolean);
{ displays/hides all Donkey Kong's chains sprites                              }
  var
    i: integer;
begin
  for i := 1 to 4 do
    aIm_kong_chain[i].Visible := b;
end;





procedure TForm1.Vid_Show_All_Keys(b: boolean);
{ displays/hides all keys sprites                                              }
  var
    i: integer;
begin
  for i := 1 to 2 do
    aIm_key[i].Visible := b;
end;





procedure TForm1.Vid_Show_All_Miss(b: boolean);
{ displays/hides all misses sprites                                            }
  var
    i: integer;
begin
  for i := 1 to 2 do
    aIm_miss[i].Visible := b;
end;





procedure TForm1.Vid_Show_Panel_Miss(var p: integer);
{ displays misses panel in various configurations                              }
  var
    i: integer;
begin
  aIm_panel_miss[0].Visible := p > 0;                                           // displays/hides item 0

  for i := 1 to 3 do
    aIm_panel_miss[i].Visible := (i <= p);                                      // displays items 1 to p, hides the remaining ones
end;





procedure TForm1.Vid_Show_Panel_Score(var p: integer; b: boolean);
{ transforms score/high-score into sprites, and displays/hides them            }
  var
    i, t: integer;
begin
  t := p;

  while t >= 1000 do                                                            // displays "1000" as "...0"
    Dec(t, 1000);

  aIm_panel_digit_pos[1].Picture := nil;

  if b then                                                                     // display or hide score/high-score?
    begin                                                                         // display
      aIm_panel_digit_pos[2].Picture := aIm_panel_number[t div 100 mod 10].Picture; // hundreds
      aIm_panel_digit_pos[3].Picture := aIm_panel_number[t div  10 mod 10].Picture; // tens
      aIm_panel_digit_pos[4].Picture := aIm_panel_number[t         mod 10].Picture; // units
    end
  else                                                                            // hide
    for i := 2 to 4 do
      aIm_panel_digit_pos[i].Picture := nil;
end;





procedure TForm1.Vid_Show_Hero(b: boolean);
{ displays hero in [iPosHeroX, iPosHeroY]                                      }
begin
  aIm_junior[iPosHeroX, iPosHeroY].Visible := b;
end;





procedure TForm1.Vid_Show_Coconut(b: boolean);
{ displays coconut in [iLoopCoconut]                                           }
begin
  aIm_coconut[iLoopCoconut].Visible := b;
end;





procedure TForm1.Vid_Show_Kong_Arm_Lock(b: boolean);
{ displays Kong's arm and lock at a certain position                           }
begin
  case iLoopKongArm of
 1, 3: begin aIm_kong_arm[4].Visible := b; aIm_kong_lock[4].Visible := b; end; // 1 is used in first loop only
 2, 4: begin aIm_kong_arm[3].Visible := b; aIm_kong_lock[3].Visible := b; end; // 2 is used in first loop only
    5: begin aIm_kong_arm[2].Visible := b; aIm_kong_lock[2].Visible := b; end;
    6: begin aIm_kong_arm[3].Visible := b; aIm_kong_lock[3].Visible := b; end;
  end; // case..of
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: AUDIO MANAGEMENT                                             * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Snd_Play(pfs: PfSoundSample);
{ plays a certain sound sample                                                 }
begin
  if bSound then
    FSOUND_PlaySound(FSOUND_FREE, pfs);
end;





procedure TForm1.Snd_Silence;
{ stops any currently playing sound sample                                     }
begin
  if bSound then
    FSOUND_StopSound(FSOUND_ALL);
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: INPUT MANAGEMENT - EVENTS                                    * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.im_backgroundMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ dragging the main form: PRESS                                                }
{ Note: this routine works if the game is not in 'Fullscreen' mode only        }
begin
{}if (not bFullscreenMode)
    and (Button = mbLeft) then
    begin
      iMouseX := X;
      iMouseY := Y;
    end;
end;





procedure TForm1.im_backgroundMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
{ dragging the main form: MOVE                                                 }
{ Note: this routine works if the game is not in 'Fullscreen' mode only        }
begin
{}if (not bFullscreenMode)
    and (ssLeft in Shift) then
    begin
      Top  := Top  + Y - iMouseY;
      Left := Left + X - iMouseX;
    end;
end;





procedure TForm1.btn_game_a_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ Game A button: PRESS                                                         }
begin
  if Button = mbLeft then
    Inp_game_a_down;
end;





procedure TForm1.btn_game_a_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ Game A button: RELEASE                                                       }
begin
  if Button = mbLeft then
    Inp_game_a_up;
end;





procedure TForm1.btn_game_b_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ Game B button: PRESS                                                         }
begin
  if Button = mbLeft then
    Inp_game_b_down;
end;





procedure TForm1.btn_game_b_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ Game B button: RELEASE                                                       }
begin
  if Button = mbLeft then
    Inp_game_b_up;
end;





procedure TForm1.btn_time_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ Time button: PRESS                                                           }
begin
  if Button = mbLeft then
    Inp_time_down;
end;





procedure TForm1.btn_time_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ Time button: RELEASE                                                         }
begin
  if Button = mbLeft then
    Inp_time_up;
end;





procedure TForm1.btn_acl_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ ACL button: PRESS                                                            }
begin
  if Button = mbLeft then
    Inp_acl_down;
end;





procedure TForm1.btn_acl_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ ACL button: RELEASE                                                          }
begin
  if Button = mbLeft then
    Inp_acl_up;
end;





procedure TForm1.btn_jump_topMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ JUMP button: PRESS                                                           }
begin
  if Button = mbLeft then
    Inp_jump_down;
end;





procedure TForm1.btn_jump_topMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ JUMP button: RELEASE                                                         }
begin
  if Button = mbLeft then
    Inp_jump_up;
end;





procedure TForm1.btn_ctrlr_overlay_upMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ UP button: PRESS                                                             }
begin
  if Button = mbLeft then
    Inp_up_down;
end;





procedure TForm1.btn_ctrlr_overlay_upMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ UP button: RELEASE                                                           }
begin
  if Button = mbLeft then
    Inp_up_up;
end;





procedure TForm1.btn_ctrlr_overlay_leftMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ LEFT button: PRESS                                                           }
begin
  if Button = mbLeft then
    Inp_left_down;
end;





procedure TForm1.btn_ctrlr_overlay_leftMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ LEFT button: RELEASE                                                         }
begin
  if Button = mbLeft then
    Inp_left_up;
end;





procedure TForm1.btn_ctrlr_overlay_rightMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ RIGHT button: PRESS                                                          }
begin
  if Button = mbLeft then
    Inp_right_down;
end;





procedure TForm1.btn_ctrlr_overlay_rightMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ RIGHT button: RELEASE                                                        }
begin
  if Button = mbLeft then
    Inp_right_up;
end;





procedure TForm1.btn_ctrlr_overlay_downMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ DOWN button: PRESS                                                           }
begin
  if Button = mbLeft then
    Inp_down_down;
end;





procedure TForm1.btn_ctrlr_overlay_downMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ DOWN button: RELEASE                                                         }
begin
  if Button = mbLeft then
    Inp_down_up;
end;





procedure TForm1.FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
{ keypress: PRESS                                                              }
begin
  // note: can't use the "case..of" construct on the aKeys[n].word
  if Key = aKeys[1].word then Inp_game_a_down else                              // Game A
  if Key = aKeys[2].word then Inp_game_b_down else                              // Game B
  if Key = aKeys[3].word then Inp_time_down   else                              // Time
  if Key = aKeys[4].word then Inp_acl_down    else                              // ACL

  if Key = aKeys[5].word then Inp_up_down     else                              // Up
  if Key = aKeys[6].word then Inp_left_down   else                              // Left
  if Key = aKeys[7].word then Inp_right_down  else                              // Right
  if Key = aKeys[8].word then Inp_down_down   else                              // Down

  if Key = aKeys[9].word then Inp_jump_down;                                    // Jump
end;





procedure TForm1.FormKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
{ keypress: RELEASE                                                            }
begin
  // note: can't use the "case..of" construct on the aKeys[n].word
  if Key = aKeys[1].word then Inp_game_a_up else                                // Game A
  if Key = aKeys[2].word then Inp_game_b_up else                                // Game B
  if Key = aKeys[3].word then Inp_time_up   else                                // Time
  if Key = aKeys[4].word then Inp_acl_up    else                                // ACL

  if Key = aKeys[5].word then Inp_up_up     else                                // Up
  if Key = aKeys[6].word then Inp_left_up   else                                // Left
  if Key = aKeys[7].word then Inp_right_up  else                                // Right
  if Key = aKeys[8].word then Inp_down_up   else                                // Down

  if Key = aKeys[9].word then Inp_jump_up;                                      // Jump
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: INPUT MANAGEMENT - ACTIONS                                   * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Inp_game_a_down;
{ Game A button: PRESS                                                         }
begin
  btn_game_a_down.Visible := true;

  if (iMode = 2) or                                                             // ACL/2     or
     (iMode = 4) or                                                             // time      or
     (iMode = 9) then                                                           // game over ->
    begin
      iMode := 5; Gam_Set_Mode;                                                   //         -> hiscore A
    end;
end;





procedure TForm1.Inp_game_a_up;
{ Game A button: RELEASE                                                       }
begin
  btn_game_a_down.Visible := false;

  if iMode = 5 then                                                             // hiscore A ->
    begin
      iMode := 6; Gam_Set_Mode;                                                   //         -> game A
    end;
end;





procedure TForm1.Inp_game_b_down;
{ Game B button: PRESS                                                         }
begin
  btn_game_b_down.Visible := true;

  if (iMode = 2) or                                                             // ACL/2     or
     (iMode = 4) or                                                             // time      or
     (iMode = 9) then                                                           // game over ->
    begin
      iMode := 7; Gam_Set_Mode;                                                   //         -> hiscore B
    end;
end;





procedure TForm1.Inp_game_b_up;
{ Game B button: RELEASE                                                       }
begin
  btn_game_b_down.Visible := false;

  if iMode = 7 then                                                             // hiscore B ->
    begin
      iMode := 8; Gam_Set_Mode;                                                   //         -> game B
    end;
end;





procedure TForm1.Inp_time_down;
{ Time button: PRESS                                                           }
begin
  btn_time_down.Visible := true;

  if (iMode = 2) or                                                             // ACL/2     or
     (iMode = 4) or                                                             // time      or
     (iMode = 9) then                                                           // game over ->
    begin
      iMode := 3; Gam_Set_Mode;                                                   //         -> alarm
    end;
end;





procedure TForm1.Inp_time_up;
{ Time button: RELEASE                                                         }
begin
  btn_time_down.Visible := false;

  if iMode = 3 then                                                             // alarm ->
    begin
      iMode := 4; Gam_Set_Mode;                                                   //     -> time
    end;
end;





procedure TForm1.Inp_acl_down;
{ ACL button: PRESS                                                            }
begin
  btn_acl_down.Visible := true;

  if iMode <> 0 then                                                            // any mode except closed ->
    begin
      iMode := 1; Gam_Set_Mode;                                                   //                      -> ACL/1
    end;
end;





procedure TForm1.Inp_acl_up;
{ ACL button: RELEASE                                                          }
begin
  btn_acl_down.Visible := false;

  if iMode = 1 then                                                             // ACL/1 ->
    begin
      iMode := 2; Gam_Set_Mode;                                                   //     -> ACL/2
    end;
end;





procedure TForm1.Inp_up_down;
{ UP joystick: PRESS                                                           }
begin
  btn_joystick_bottom.Picture := btn_joystick_bottom_up.Picture;

  if ( (iMode = 6) or (iMode = 8) ) and bControls and (not bKeyPressed) then
    begin
      bKeypressed := true;

      if (iPosHeroX = 1) and (iPosHeroY = 1) then
        begin
          aIm_junior_hand_1[1].Visible := false;
          aIm_junior_hand_1[2].Visible := true;

          if aIm_coconut[0].Visible then
            begin
              Vid_Show_Coconut(false);

              Snd_Silence;
              Snd_Play(pfs_coconut);

              iScoreCoconut := 2;
              iLoopCoconut := 0;
              Timer_Coconut.Interval := 1;
              Timer_Coconut.Enabled := true;

              Timer_Coconut_Music.Interval := 1900;
              Timer_Coconut_Music.Enabled := true;
            end
          else
            Snd_Play(pfs_hand1);

          iLoopJunior11Hand := 1;
          Timer_Junior_1_1_Hand.Interval := 200;
          Timer_Junior_1_1_Hand.Enabled := true;
      end;

      if (iPosHeroX = 1) and (iPosHeroY = 2) then
        begin
          Vid_Show_Hero(false);
          Dec(iPosHeroY);
          Vid_Show_Hero(true);
          aIm_junior_hand_1[1].Visible := true;

          Snd_Play(pfs_step);

          Timer_Junior_Step.Interval := 466;
          Timer_Junior_Step.Enabled := true;
        end;

      if (iPosHeroX = 3) and (iPosHeroY = 2) then
        begin
          Vid_Show_Hero(false);
          Dec(iPosHeroY);
          Vid_Show_Hero(true);

          Snd_Play(pfs_step);

          Timer_Junior_Step.Interval := 466;
          Timer_Junior_Step.Enabled := true;
        end;
    end;
end;





procedure TForm1.Inp_up_up;
{ UP joystick: RELEASE                                                         }
begin
  btn_joystick_bottom.Picture := nil;

  bKeypressed := false;
end;





procedure TForm1.Inp_down_down;
{ DOWN joystick: PRESS                                                         }
begin
  btn_joystick_bottom.Picture := btn_joystick_bottom_down.Picture;

  if ( (iMode = 6) or (iMode = 8) ) and bControls and (not bKeyPressed) then
    begin
      bKeypressed := true;

      if (iPosHeroX = 1) and (iPosHeroY = 1) then
        begin
          aIm_junior_hand_1[1].Visible := false;
          aIm_junior_hand_1[2].Visible := false;
        end;

      if   ( (iPosHeroX = 1) and (iPosHeroY = 2) )
        or ( (iPosHeroX = 1) and (iPosHeroY = 1) )
        or ( (iPosHeroX = 2) and (iPosHeroY = 3) )
        or ( (iPosHeroX = 3) and (iPosHeroY = 2) )
        or ( (iPosHeroX = 3) and (iPosHeroY = 1) )
        or ( (iPosHeroX = 3) and (iPosHeroY = 4) )
        or ( (iPosHeroX = 4) and (iPosHeroY = 3) ) then
        begin
          Vid_Show_Hero(false);
          Inc(iPosHeroY);
          Vid_Show_Hero(true);

          Snd_Play(pfs_step);
        end;

      Timer_Junior_Step.Interval := 466;
      Timer_Junior_Step.Enabled := true;
    end;
end;





procedure TForm1.Inp_down_up;
{ DOWN joystick: RELEASE                                                       }
begin
  btn_joystick_bottom.Picture := nil;

  bKeypressed := false;
end;





procedure TForm1.Inp_left_down;
{ LEFT joystick: PRESS                                                         }
begin
  btn_joystick_bottom.Picture := btn_joystick_bottom_left.Picture;

  if ( (iMode = 6) or (iMode = 8) ) and bControls and (not bKeyPressed) then
    begin
      bKeypressed := true;

      if   ( (iPosHeroX = 2) and (iPosHeroY = 1) )
        or ( (iPosHeroX = 2) and (iPosHeroY = 3) )
        or ( (iPosHeroX = 2) and (iPosHeroY = 5) )
        or ( (iPosHeroX = 3) and (iPosHeroY = 1) )
        or ( (iPosHeroX = 3) and (iPosHeroY = 5) )
        or ( (iPosHeroX = 4) and (iPosHeroY = 1) )
        or ( (iPosHeroX = 4) and (iPosHeroY = 3) ) then
        begin
          Vid_Show_Hero(false);
          Dec(iPosHeroX);
          Vid_Show_Hero(true);

          Snd_Play(pfs_step);

          if Gam_Check_Collisions then
            Gam_Execute_Miss_Crow;
        end;

      if   ( (iPosHeroX = 2) and (iPosHeroY = 2) )
        or ( (iPosHeroX = 3) and (iPosHeroY = 2) )
        or ( (iPosHeroX = 3) and (iPosHeroY = 4) ) then
        if Gam_Check_Collisions then
          Gam_Execute_Miss_Crow
        else
          begin
            Vid_Show_Hero(false);
            Dec(iPosHeroX);
            Vid_Show_Hero(true);

            Snd_Play(pfs_step);
          end;

      if   ( (iPosHeroX = 5) and (iPosHeroY = 1) )
        or ( (iPosHeroX = 5) and (iPosHeroY = 4) ) then
        begin
          Vid_Show_Hero(false);
          Dec(iPosHeroX);
          Vid_Show_Hero(true);

          Snd_Silence;
          Snd_Play(pfs_step);

          Timer_Flying_Music.Enabled := false;
        end;

      if (iPosHeroX = 6) then
        begin
          Vid_Show_Hero(false);
          Dec(iPosHeroX);
          Vid_Show_Hero(true);

          if (iPosHeroY > 1) then
            aIm_junior_hand_6[iPosHeroY].Visible := false;

          if aIm_umbrella[iPosHeroY].Visible then
            begin
              Inc(iScore, iScoreMultiplier);
              Vid_Show_Panel_Score(iScore, true);
              Gam_Set_Difficulty;

              iChanceStatus := 1;
              Gam_Check_Chance_Time;
            end
          else
            Gam_Execute_Miss_Fall;
        end;

      // after moving
      if (iPosHeroX = 1) and (iPosHeroY = 1) then
        aIm_junior_hand_1[1].Visible := true;

      if (iPosHeroX = 4) and (iPosHeroY = 1) and Gam_Check_Collisions then
        Gam_Execute_Miss_Crow;

      Timer_Junior_Step.Interval := 466;
      Timer_Junior_Step.Enabled := true;
    end;
end;





procedure TForm1.Inp_left_up;
{ LEFT joystick: RELEASE                                                       }
begin
  btn_joystick_bottom.Picture := nil;

  bKeypressed := false;
end;





procedure TForm1.Inp_right_down;
{ RIGHT joystick: PRESS                                                        }
begin
  btn_joystick_bottom.Picture := btn_joystick_bottom_right.Picture;

  if ( (iMode = 6) or (iMode = 8) ) and bControls and (not bKeyPressed) then
    begin
      bKeypressed := true;

      if (iPosHeroX = 6) and (iPosHeroY > 1) then
        begin
          aIm_junior_hand_6[iPosHeroY].Visible := true;

          Timer_Flying_Music.Enabled := false;
          Snd_Silence;

          if (iKeyStatus = 2) then
            if aIm_kong_lock[iPosHeroY].Visible then
              begin
                Gam_Stop_All_Timers;

                bControls := false;

                Snd_Play(pfs_bonus1);

                iLoopBonusPhase1 := 1;
                Timer_Bonus_Phase_1.Interval := 1;
                Timer_Bonus_Phase_1.Enabled := true;
              end
            else
              begin
                Gam_Stop_All_Timers;
                bControls := false;
                Snd_Play(pfs_key2);

                iLoopKeyFall := 1;
                Timer_Key_Fall.Interval := 100;
                Timer_Key_Fall.Enabled := true;
              end
            else
              begin
                Snd_Play(pfs_hand2);

                Timer_Junior_6_Hand.Interval := 240;
                Timer_Junior_6_Hand.Enabled := true;
              end;
        end;

      if (iPosHeroX = 5) then
        begin
          Vid_Show_Hero(false);
          Inc(iPosHeroX);
          Vid_Show_Hero(true);

          if aIm_balloon[iPosHeroY].Visible then
            begin
              Inc(iScore, iScoreMultiplier);
              Vid_Show_Panel_Score(iScore, true);
              Gam_Set_Difficulty;

              iChanceStatus := 1;
              Gam_Check_Chance_Time;
            end
          else
            Gam_Execute_Miss_Fall;
        end;

      if (iPosHeroX = 4) and (iPosHeroY = 4) then
        begin
          Vid_Show_Hero(false);
          Inc(iPosHeroX);
          Vid_Show_Hero(true);

          Snd_Play(pfs_step);

          if aIm_umbrella[4].Visible then
            begin
              Timer_Coconut_Music.Enabled := false;

              Snd_Silence;
              Snd_Play(pfs_flying);
              Timer_Flying_Music.Interval := 4060;
              Timer_Flying_Music.Enabled := true;

              Inc(iScore, iScoreMultiplier);
              Vid_Show_Panel_Score(iScore, true);
              Gam_Set_Difficulty;

              iChanceStatus := 1;
              Gam_Check_Chance_Time;
            end
          else
            Gam_Execute_Miss_Fall;
        end;

      if   ( (iPosHeroX = 2) and (iPosHeroY = 2) )
        or ( (iPosHeroX = 2) and (iPosHeroY = 4) )
        or ( (iPosHeroX = 1) and (iPosHeroY = 2) ) then
        begin
          Vid_Show_Hero(false);
          Inc(iPosHeroX);
          Vid_Show_Hero(true);

          Snd_Play(pfs_step);

          if Gam_Check_Collisions then
            Gam_Execute_Miss_Crow;
        end;

      if (iPosHeroX = 4) and (iPosHeroY = 1) then
        if Gam_Check_Collisions then
          Gam_Execute_Miss_Crow
        else
          begin
            Vid_Show_Hero(false);
            Inc(iPosHeroX);
            Vid_Show_Hero(true);

            Snd_Play(pfs_step);

            if aIm_umbrella[1].Visible then
              begin
                Timer_Coconut_Music.Enabled := false;

                Snd_Silence;
                Snd_Play(pfs_flying);
                Timer_Flying_Music.Interval := 4060;
                Timer_Flying_Music.Enabled := true;

                Inc(iScore, iScoreMultiplier);
                Vid_Show_Panel_Score(iScore, true);
                Gam_Set_Difficulty;

                iChanceStatus := 1;
                Gam_Check_Chance_Time;
              end
            else
              Gam_Execute_Miss_Fall;
          end;

      if (iPosHeroX = 1) and (iPosHeroY = 1) then
        if Gam_Check_Collisions then
          Gam_Execute_Miss_Crow
        else
          begin
            Vid_Show_Hero(false);
            Inc(iPosHeroX);
            Vid_Show_Hero(true);
            aIm_junior_hand_1[1].Visible := false;
            aIm_junior_hand_1[2].Visible := false;

            Snd_Play(pfs_step);
          end
      else
        if   ( (iPosHeroX = 3) and (iPosHeroY = 1) )
          or ( (iPosHeroX = 3) and (iPosHeroY = 3) )
          or ( (iPosHeroX = 2) and (iPosHeroY = 1) )
          or ( (iPosHeroX = 2) and (iPosHeroY = 5) )
          or ( (iPosHeroX = 1) and (iPosHeroY = 3) )
          or ( (iPosHeroX = 1) and (iPosHeroY = 5) ) then
            if Gam_Check_Collisions then
              Gam_Execute_Miss_Crow
            else
              begin
                Vid_Show_Hero(false);
                Inc(iPosHeroX);
                Vid_Show_Hero(true);

                Snd_Play(pfs_step);
              end;

      Timer_Junior_Step.Interval := 466;
      Timer_Junior_Step.Enabled := true;
    end;
end;





procedure TForm1.Inp_right_up;
{ RIGHT joystick: RELEASE                                                      }
begin
  btn_joystick_bottom.Picture := nil;

  bKeypressed := false;
end;





procedure TForm1.Inp_jump_down;
{ JUMP button: PRESS                                                           }
begin
  btn_jump_down.Visible := true;

  if ( (iMode = 6) or (iMode = 8) ) and bControls and (not bKeyPressed) then
    begin
      bKeypressed := true;

      if   ( (iPosHeroX = 1) and (iPosHeroY = 3) )
        or ( (iPosHeroX = 2) and (iPosHeroY = 4) )
        or ( (iPosHeroX = 3) and (iPosHeroY = 3) )
        or ( (iPosHeroX = 3) and (iPosHeroY = 5) )
        or ( (iPosHeroX = 4) and (iPosHeroY = 4) )
        then begin
          Vid_Show_Hero(false);
          Dec(iPosHeroY);
          Vid_Show_Hero(true);

          Snd_Play(pfs_jump);
        end;

      if (iPosHeroX = 1) and (iPosHeroY = 5) then
        begin
          Vid_Show_Hero(false);
          Dec(iPosHeroY);
          Vid_Show_Hero(true);

          if iKeyStatus = 1 then
            Snd_Play(pfs_key1)
          else
            Snd_Play(pfs_jump);

          bControls := false;

          iLoopJuniorJump := 1;
          Timer_Junior_Jump.Interval := 1;
          Timer_Junior_Jump.Enabled := true;
        end;
    end;
end;





procedure TForm1.Inp_jump_up;
{ JUMP button: RELEASE                                                         }
begin
  btn_jump_down.Visible := false;

  bKeypressed := false;
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: GAME                                                         * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Gam_Stop_All_Timers;
{ stops all currently enabled timers                                           }
begin
  Timer_Bonus_Phase_1.Enabled    := false;
  Timer_Bonus_Phase_2.Enabled    := false;
  Timer_Chance.Enabled           := false;
  Timer_Clock.Enabled            := false;
  Timer_Coconut.Enabled          := false;
  Timer_Coconut_Bonus.Enabled    := false;
  Timer_Coconut_Music.Enabled    := false;
  Timer_Flying_Music.Enabled     := false;
  Timer_Game.Enabled             := false;
  Timer_Game_Over.Enabled        := false;
  Timer_Game_Start.Enabled       := false;
  Timer_Junior_1_1_Hand.Enabled  := false;
  Timer_Junior_6_Hand.Enabled    := false;
  Timer_Junior_Jump.Enabled      := false;
  Timer_Junior_Step.Enabled      := false;
  Timer_Key_Blink.Enabled        := false;
  Timer_Key_Fall.Enabled         := false;
  Timer_Kong_Arm.Enabled         := false;
  Timer_Miss_Crow.Enabled        := false;
  Timer_Miss_Fall.Enabled        := false;
  Timer_Score_Blink.Enabled      := false;
  Timer_Umbrella_Balloon.Enabled := false;
end;





procedure TForm1.Gam_Check_Chance_Time;
{ stops action, sets the score value for the next "chance time", and executes  }
{ the "chance time" intermission"                                              }
begin
  if ( (iMode = 6) or (iMode = 8) )
    and (iScore >= iScoreNextChance)
    and (not bChance) then
      begin
        Gam_Stop_All_Timers;
        Snd_Silence;

        bControls := false;

        Inc(iScoreNextChance, 1000);

        bChance := (iMiss = 0);   // iMiss=0 -> true; iMiss=1 or 2 -> false;
        iLoopChance := 3 - iMiss; // iMiss=0 -> iLoopChance=3
        Timer_Chance.Interval := 500;
        Timer_Chance.Enabled := true;
      end;
end;





function TForm1.Gam_Check_Collisions: boolean;
{ returns "true" if a collision is detected between Junior and a crow          }
begin
  result := false;

  case iPosHeroY of
    1: if (iPosHeroX < 5) and aIm_crow[iPosHeroX + 1, iPosHeroY].Visible then result := true;
    2: if (iPosHeroX < 4) and aIm_crow[iPosHeroX,     iPosHeroY].Visible then result := true;
    3: if (iPosHeroX < 5) and aIm_crow[iPosHeroX + 1, iPosHeroY].Visible then result := true;
    4: if (iPosHeroX < 5) and aIm_crow[iPosHeroX,     iPosHeroY].Visible then result := true;
    5: if (iPosHeroX < 4) and aIm_crow[iPosHeroX + 1, iPosHeroY].Visible then result := true;
  end; // case..of

  if (iPosHeroX = 1) and (iPosHeroY = 5)
    and aIm_crow[2, 5].Visible and aIm_crow[1, 4].Visible then
    result := false; // to prevent Junior from colliding with crow at (2,5) when a crow is flying over him at (1,4)  
end;





procedure TForm1.Gam_Execute_Miss_Crow;
{ a miss has occured because of a crow                                         }
begin
  Gam_Stop_All_Timers;
  Snd_Silence;

  bControls := false;

  Vid_Show_Panel_Score(iScore, true);

  Snd_Play(pfs_miss);

  iLoopMissCrow := 1;
  Timer_Miss_Crow.Interval := 1;
  Timer_Miss_Crow.Enabled := true;
end;





procedure TForm1.Gam_Execute_Miss_Fall;
{ a miss has occured because of a fall                                         }
begin
  Gam_Stop_All_Timers;
  Snd_Silence;

  bControls := false;

  Vid_Show_Panel_Score(iScore, true);

  Snd_Play(pfs_falling);

  iLoopMissFall := iPosHeroY;
  Timer_Miss_Fall.Interval := 1;
  Timer_Miss_Fall.Enabled := true;
end;





procedure TForm1.Gam_Reprise_After_Miss;
{ game reprise after miss                                                      }
begin
  Inc(iMiss); Vid_Show_Panel_Miss(iMiss);

  if iMiss < 3 then
    // game continues
    begin
      iPosHeroX := 1; iPosHeroY := 5; Vid_Show_Hero(true);

      aIm_crow[2, 5].Visible := false;
      aIm_crow[3, 5].Visible := false;

      Vid_Show_All_Coconuts(false);

      iScoreMultiplier := 1;
      bChance := false;

      Gam_Set_Difficulty;
      Timer_Game.Enabled := true;

      // no need to update iLoopKongArm
      Timer_Kong_Arm.Interval := 740;
      Timer_Kong_Arm.Enabled := true;

      // no need to update iLoopUmbrellaBalloon
      Timer_Umbrella_Balloon.Interval := 800;
      Timer_Umbrella_Balloon.Enabled := true;

      iKeyStatus := 1;
      aIm_key[1].Visible := true;
      Timer_Key_Blink.Enabled := true;

      bControls := true;
    end
  else
    // game is over
    begin
      while iScore >= 1000 do
        Dec(iScore, 1000);

      if (iMode = 6) and (iScore > iHiScoreA) then
        begin
          iHiScoreA := iScore; iReg.WriteInteger('', 'HiScore1', iHiScoreA);
        end;

      if (iMode = 8) and (iScore > iHiScoreB) then
        begin
          iHiScoreB := iScore; iReg.WriteInteger('', 'HiScore2', iHiScoreB);
        end;

      iMode := 9;

      // Junior blinks for 5 minutes
      iLoopGameOver := 1;
      Timer_Game_Over.Interval := 1000;
      Timer_Game_Over.Enabled := true;
    end;
end;





procedure TForm1.Gam_Set_Difficulty;
{ calculates difficulty increase, depending on current score                   }
  var
    iUnits,
    iHundreds: integer;
begin
  iUnits := iScore;
  iHundreds := 0;

  while iUnits > 99 do
    begin
      Dec(iUnits, 100);
      Inc(iHundreds);
    end;

  case iMode of
    6: begin                                                                    // Game A
         // Timer_Game.Interval
         case iUnits of
            0..19: Timer_Game.Interval := 1000;
           20..39: Timer_Game.Interval := 930;
           40..59: Timer_Game.Interval := 860;
           60..79: Timer_Game.Interval := 790;
           80..99: Timer_Game.Interval := 720;
         end; // case..of

         if iHundreds <= 9 then // score < 999
           Timer_Game.Interval := Integer(Timer_Game.Interval) - (iHundreds * 20)
         else                   // score > 1000
           Timer_Game.Interval := Integer(Timer_Game.Interval) - 200;

         // iMaxRandom
         case iUnits of
            0..19: iMaxRandom := 20;
           20..39: iMaxRandom := 14;
           40..79: iMaxRandom := 8;
           80..99: iMaxRandom := 6;
         end; // case..of

         if iScore > 99 then
           Dec(iMaxRandom, 1);
       end;
    8: begin                                                                    // Game B
         // Timer_Game.Interval
         case iUnits of
            0..19: Timer_Game.Interval := 870;
           20..39: Timer_Game.Interval := 810;
           40..59: Timer_Game.Interval := 730;
           60..79: Timer_Game.Interval := 670;
           80..99: Timer_Game.Interval := 600;
         end; // case..of

         if (iHundreds >= 2) and (iHundreds <= 9) then // score 200-999
           Timer_Game.Interval := Integer(Timer_Game.Interval) - 40 - (iHundreds * 30);

         if iHundreds >= 10 then                       // score > 1000
           Timer_Game.Interval := Timer_Game.Interval - 340;

         // iMaxRandom
         case iUnits of
            0..19: iMaxRandom := 12;
           20..59: iMaxRandom := 8;
           60..79: iMaxRandom := 7;
           80..99: iMaxRandom := 6;
         end; // case..of
       end;
  end; // case..of
end;





procedure TForm1.Gam_Set_Mode;
{ sets game/time/ACL mode, depending on the value of the iMode variable        }
begin
  case iMode of
    0: begin                                                                    // handheld is closed
       end;
    1: begin                                                                    // ACL/1 (ACL button press)
         Gam_Stop_All_Timers;
         Snd_Silence;

         bControls := false;
       end;
    2: Vid_Show_All(true);                                                      // ACL/2 (ACL button release)
    3: begin                                                                    // ALARM (Time button press)
         Gam_Stop_All_Timers;

         Vid_Show_All(false);
         im_game_background.Visible := true;
         aIm_Panel_Alarm[1].Visible := true;
         aIm_Panel_Alarm[2].Visible := true;
         im_panel_time_pm.Visible := true;
         im_panel_time_colon.Visible := true;
         aIm_panel_digit_pos[1].Picture := aIm_panel_number[1].Picture;
         aIm_panel_digit_pos[2].Picture := aIm_panel_number[2].Picture;
         aIm_panel_digit_pos[3].Picture := aIm_panel_number[0].Picture;
         aIm_panel_digit_pos[4].Picture := aIm_panel_number[0].Picture;

         bControls := false;
       end;
    4: begin                                                                    // TIME (Time button release)
         Gam_Stop_All_Timers;

         Vid_Show_All(false);
         im_game_background.Visible := true;
         aIm_Panel_Alarm[1].Visible := true;
         aIm_Panel_Alarm[2].Visible := false;
         im_panel_time_colon.Visible := true;

         Timer_Clock.Interval := 1;
         Timer_Clock.Enabled := true;

         bControls := false;
       end;
 5, 7: begin                                                                    // HISCORE A/B (Game A/B button press)
         Gam_Stop_All_Timers;

         Vid_Show_All(false);

         case iMode of
           5: begin im_panel_game_a.Visible := true; Vid_Show_Panel_Score(iHiScoreA, true); end;
           7: begin im_panel_game_b.Visible := true; Vid_Show_Panel_Score(iHiScoreB, true); end;
         end; // case..of

         bControls := false;
       end;
 6, 8: begin                                                                    // GAME A/B (Game A/B button release)
         im_game_background.Visible := true;
         aIm_panel_alarm[1].Visible := true;
         aIm_panel_alarm[2].Visible := false;
         aIm_panel_alarm[3].Visible := false;
         iPosHeroX := 1; iPosHeroY := 5; Vid_Show_Hero(true);
         im_kong_body.Visible := true; im_kong_arm_r1.Visible := true;

         iLoopKongArm := 1; Vid_Show_Kong_Arm_Lock(true);

         iMiss := 0;
         iScore := 0; Vid_Show_Panel_Score(iScore, true);
         iScoreMultiplier := 1;
         iScoreNextChance := 300;

         iChainsRemoved := 0; Vid_Show_All_Kong_Chains(true);

         bChance := false;

         iLoopGameStart := 1;
         Timer_Game_Start.Interval := 1;
         Timer_Game_Start.Enabled := true;

         iKeyStatus := 1;
         aIm_key[1].Visible := true;
         Timer_Key_Blink.Interval := 210;
         Timer_Key_Blink.Enabled := true;

         bKeypressed := false;
       end;
    9: begin                                                                    // GAME OVER
         // does nothing
       end;
    end; // case..of
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: TIMERS                                                       * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.Timer_Bonus_Phase_1Timer(Sender: TObject);
{ a lock has been unlocked                                                     }
begin
  case iLoopBonusPhase1 of
    1: begin
         Inc(iChainsRemoved);

         Timer_Bonus_Phase_1.Interval := 180;
       end;
    2, 4, 6, 8: aIm_kong_chain[iChainsRemoved].Visible := false;
    3, 5, 7, 9: aIm_kong_chain[iChainsRemoved].Visible := true;
   10: begin
         aIm_kong_chain[iChainsRemoved].Visible := false;
         Vid_Show_Kong_Arm_Lock(false);
         im_kong_arm_l2.Visible := true;
         aIm_junior_hand_6[iPosHeroY].Visible := false;

         if iChainsRemoved = 4 then
           begin
             Vid_Show_Hero(false);
             im_junior_victory.Visible := true;
           end;

         Timer_Bonus_Phase_1.Interval := 160;
       end;
   11, 12, 13, 14, 15, 16, 17, 18, 19, 20: begin
         Inc(iScore, iScoreMultiplier);
         Vid_Show_Panel_Score(iScore, true);
         Gam_Set_Difficulty;

         iChanceStatus := 3;
         Gam_Check_Chance_Time;

         Snd_Play(pfs_point);
       end;
   21: if iChainsRemoved = 4 then
         begin
           Timer_Bonus_Phase_1.Enabled := false;

           iLoopBonusPhase2 := 1;
           Timer_Bonus_Phase_2.Interval := 1;
           Timer_Bonus_Phase_2.Enabled := true;
         end
       else
         Timer_Bonus_Phase_1.Interval := 900;
   22: begin
         im_kong_arm_l2.Visible := false;

         iLoopKongArm := 5;
         Vid_Show_Kong_Arm_Lock(true);

         iKeyStatus := 1;
         aIm_key[1].Visible := true;
         Timer_Key_Blink.Enabled := true;

         Timer_Umbrella_Balloon.Interval := 800;
         Timer_Umbrella_Balloon.Enabled := true;

         Timer_Kong_Arm.Interval := 740;
         Timer_Kong_Arm.Enabled := true;

         Snd_Play(pfs_flying);
         Timer_Flying_Music.Interval := 4060;
         Timer_Flying_Music.Enabled := true;

         if bChance then
           begin
             Timer_Score_Blink.Interval := 800;
             Timer_Score_Blink.Enabled := true;
           end;

         Gam_Set_Difficulty;
         Timer_Game.Enabled := true;

         bControls := true;

         Timer_Bonus_Phase_1.Enabled := false;
       end;
  end; // case..of

  Inc(iLoopBonusPhase1);
end;





procedure TForm1.Timer_Bonus_Phase_2Timer(Sender: TObject);
{ all locks have been unlocked                                                 }
begin
  case iLoopBonusPhase2 of
    1: Timer_Bonus_Phase_2.Interval := 250;
    2: begin
         Inc(iKongsFreed);

         if iKongsFreed = 1 then
           Snd_Play(pfs_bonus2)
         else
           Snd_Play(pfs_bonus3);

         Timer_Bonus_Phase_2.Interval := 800;
       end;
    3, 5, 7, 9: begin
         im_kong_arm_r1.Visible := false;
         im_kong_arm_r2.Visible := true;

         if iLoopBonusPhase2 = 3 then
           Timer_Bonus_Phase_2.Interval := 250;

         if iLoopBonusPhase2 = 9 then
           Timer_Bonus_Phase_2.Interval := 160;
       end;
    4, 6, 8: begin
         im_kong_arm_r2.Visible := false;
         im_kong_arm_r1.Visible := true;
       end;
   10, 11, 12, 13, 14, 15, 16, 17, 18, 19: begin
         Inc(iScore, iScoreMultiplier);
         Vid_Show_Panel_Score(iScore, true);
         Gam_Set_Difficulty;

         iChanceStatus := 4;
         Gam_Check_Chance_Time;
       end;
   20: begin
         Timer_Bonus_Phase_2.Interval := 350;
       end;
   21: begin
         im_junior_victory.Visible := false;
         im_kong_arm_r2.Visible := false;
         im_kong_arm_r1.Visible := true;

         Snd_Play(pfs_start2);

         iLoopKongArm := 5; Vid_Show_Kong_Arm_Lock(true);

         iKeyStatus := 1;
         aIm_key[1].Visible := true;
         Timer_Key_Blink.Enabled := true;

         iPosHeroX := 1; iPosHeroY := 5; Vid_Show_Hero(true);

         aIm_crow[2, 5].Visible := false;
         aIm_crow[3, 5].Visible := false;

         if aIm_coconut[0].Visible then
           begin
             Vid_Show_All_Coconuts(false);
             aIm_coconut[0].Visible := true;
           end
         else
           Vid_Show_All_Coconuts(false);

         iChainsRemoved := 0;
         Vid_Show_All_Kong_Chains(true);

         Timer_Bonus_Phase_2.Interval := 3300;
       end;
   22: begin
         Gam_Set_Difficulty;
         Timer_Game.Enabled := true;

         Timer_Kong_Arm.Interval := 740;
         Timer_Kong_Arm.Enabled := true;

         Timer_Umbrella_Balloon.Interval := 800;
         Timer_Umbrella_Balloon.Enabled := true;

         if bChance then
           begin
             Timer_Score_Blink.Interval := 800;
             Timer_Score_Blink.Enabled := true;
           end;

         bControls := true;

         Timer_Bonus_Phase_2.Enabled := false;
       end;
  end; // case..of

  Inc(iLoopBonusPhase2);
end;





procedure TForm1.Timer_ChanceTimer(Sender: TObject);
begin
{ "chance time" intermission                                                   }
{ the procedure is called with "iLoopChance" equaled to 1, 2 or 3, depending   }
{ on the number of misses made; for this reason the Interval is set in all     }
{ cases (1, 2 and 3); case 4 activates the chance time, the doubled score and  }
{ the blinking score panel, if needed                                          }
  case iLoopChance of
    1: begin Timer_Chance.Interval := 1800; Snd_Play(pfs_chance); Dec(iMiss); Vid_Show_Panel_Miss(iMiss); end; // 2 misses
    2: begin Timer_Chance.Interval := 1800; Snd_Play(pfs_chance); Dec(iMiss); Vid_Show_Panel_Miss(iMiss); end; // 1 miss
    3: begin Timer_Chance.Interval := 1800; Snd_Play(pfs_chance); end;                                         // 0 misses
    4: begin
         if bChance then // bchance has been set in Gam_Check_Chance_Time (before calling this procedure)
           begin
            iScoreMultiplier := 2;

            Vid_Show_Panel_Score(iScore, false);

            iLoopScoreBlink := 1;
            Timer_Score_Blink.Interval := 800;
            Timer_Score_Blink.Enabled := true;
          end;

         case iChanceStatus of
           1: begin // game continues -> umbrella/balloon
                Snd_Silence;
                Snd_Play(pfs_flying);
                Timer_Flying_Music.Interval := 4060;
                Timer_Flying_Music.Enabled := true;

                Timer_Game.Enabled := true;

                Timer_Kong_Arm.Interval := 740;
                Timer_Kong_Arm.Enabled := true;

                Timer_Umbrella_Balloon.Interval := 800;
                Timer_Umbrella_Balloon.Enabled := true;

                if iKeyStatus = 1 then
                  Timer_Key_Blink.Enabled := true;

                bControls := true;

                // this is needed because player can make 1 point score (and
                // enable the "chance time" while the coconut is fallong down!)
                if aIm_coconut[0].Visible then
                  begin
                    Vid_Show_All_Coconuts(false);
                    aIm_coconut[0].Visible := true;
                  end
                else
                  Vid_Show_All_Coconuts(false);
              end;
           2: begin // game continues -> coconut falls
                Snd_Silence;
                Snd_Play(pfs_coconut);

                Timer_Coconut.Enabled := true;

                Timer_Coconut_Music.Interval := 1900;
                Timer_Coconut_Music.Enabled := true;

                Gam_Set_Difficulty;
                Timer_Game.Enabled := true;

                Timer_Kong_Arm.Interval := 740;
                Timer_Kong_Arm.Enabled:=true;

                Timer_Umbrella_Balloon.Interval := 800;
                Timer_Umbrella_Balloon.Enabled := true;

                if iKeyStatus=1 then
                  Timer_Key_Blink.Enabled := true;

                bControls := true;
              end;
           3: Timer_Bonus_Phase_1.Enabled := true; // game continues -> key bonus phase 1
           4: Timer_Bonus_Phase_2.Enabled := true; // game continues -> key bonus phase 2
         end; // case..of

         Timer_Chance.Enabled := false;
       end;
  end; // case..of

  Inc(iLoopChance);
end;




procedure TForm1.Timer_ClockTimer(Sender: TObject);
{ transforms current time into "sprites" form, then displays it                }
  var
    present:              TDateTime;
    hour, min, sec, msec: Word;
begin
  present := Now;                                                               // gets system time...
  DecodeTime(present, hour, min, sec, msec);                                    // and decodes it

  if hour > 11 then                                                             // changes mode from 0/24 to 12 AM/12 PM
    begin
      Dec(hour, 12);
      im_panel_time_am.Visible := false;
      im_panel_time_pm.Visible := true;
    end
  else
    begin
      im_panel_time_am.Visible := true;
      im_panel_time_pm.Visible := false;
    end;

  if hour = 0 then                                                              // changes mode from 0/24 to 12 AM/12 PM
    hour := 12;

  if hour > 9 then                                                              // hour: tens
    aIm_panel_digit_pos[1].Picture := aIm_panel_number[hour div 10].Picture
  else
    aIm_panel_digit_pos[1].Picture := nil;

  aIm_panel_digit_pos[2].Picture := aIm_panel_number[hour mod 10].Picture;      // hour: units
  aIm_panel_digit_pos[3].Picture := aIm_panel_number[min  div 10].Picture;      // minute: tens
  aIm_panel_digit_pos[4].Picture := aIm_panel_number[min  mod 10].Picture;      // minute: units

  Timer_Clock.Interval := 1000;
end;





procedure TForm1.Timer_CoconutTimer(Sender: TObject);
{ coconut falls down, eventually hitting crows                                 }
begin
  Vid_Show_Coconut(false);

  Inc(iLoopCoconut);

  if iLoopCoconut < 6 then
    begin
      Vid_Show_Coconut(true);

      if aIm_crow[2, iLoopCoconut].Visible then
        begin
          iScoreBonusValue := iScoreCoconut;

          Inc(iScoreCoconut, 2);

          bControls := false;

          Timer_Coconut.Enabled := false;

          iLoopCoconutBonus := 1;
          Timer_Coconut_Bonus.Interval := 1;
          Timer_Coconut_Bonus.Enabled := true;
        end;

      Timer_Coconut.Interval := 310;
    end
  else
    Timer_Coconut.Enabled := false;
end;





procedure TForm1.Timer_Coconut_BonusTimer(Sender: TObject);
{ the coconut has hit a crow: bonus intermission starts                        }
begin
  case iLoopCoconutBonus of
    1: if iScoreBonusValue = 0 then
         begin
           Timer_Coconut_Bonus.Interval := 1;

           Inc(iLoopCoconutBonus);
         end
       else
         begin
           Gam_Stop_All_Timers;
           Snd_Silence;

           Snd_Play(pfs_point);

           aIm_crow[2, iLoopCoconut].Visible := false;

           Dec(iScoreBonusValue);

           Timer_Coconut_Bonus.Interval := 200;
           Timer_Coconut_Bonus.Enabled := true; // needed!

           Inc(iScore, iLoopCoconut * iScoreMultiplier);
           Vid_Show_Panel_Score(iScore, true);
           Gam_Set_Difficulty;

           iChanceStatus := 2;
           Gam_Check_Chance_Time;
         end;
    2: begin // reprises
         Timer_Flying_Music.Enabled := false;

         Snd_Silence;
         Snd_Play(pfs_coconut);

         Timer_Coconut.Enabled := true;

         Timer_Coconut_Music.Interval := 1900;
         Timer_Coconut_Music.Enabled := true;

         Gam_Set_Difficulty;
         Timer_Game.Enabled := true;

         Timer_Kong_Arm.Interval := 740;
         Timer_Kong_Arm.Enabled := true;

         Timer_Umbrella_Balloon.Interval := 800;
         Timer_Umbrella_Balloon.Enabled := true;

         if iKeyStatus = 1 then
           Timer_Key_Blink.Enabled := true;

         bControls := true;

         Timer_Coconut_Bonus.Enabled := false;
       end;
  end; // case..of
end;





procedure TForm1.Timer_Coconut_MusicTimer(Sender: TObject);
{ falling coconut tune stops playing - eventually starts playing "flying" tune }
begin
  if (iPosHeroX > 4) and (iPosHeroY < 5) then
    begin
      Snd_Play(pfs_flying);

      Timer_Flying_Music.Interval := 4060;
      Timer_Flying_Music.Enabled := false;
      Timer_Flying_Music.Enabled := true;
    end;

  Timer_Coconut_Music.Enabled := false;
end;





procedure TForm1.Timer_Flying_MusicTimer(Sender: TObject);
{ "flying" tune plays as long as the timer is enabled                          }
begin
  Snd_Play(pfs_flying);
end;





procedure TForm1.Timer_GameTimer(Sender: TObject);
{ main game action                                                             }
  var
    i, j,
    iCrowCounter: integer;

    bGenerateCrow,
    bPlaySound: boolean;
begin
  if Gam_Check_Collisions then
    Gam_Execute_Miss_Crow
  else
    begin
      bPlaySound := false;

      case iLoopGame of
        1: begin                                                                // crows -> odd row
             { row 0 } aIm_crow[5, 1].Visible := aIm_crow[5, 0].Visible; aIm_crow[5, 0].Visible := aIm_crow[4,0].Visible;
             { row 2 } aIm_crow[5, 3].Visible := aIm_crow[5, 2].Visible; for i := 5 downto 2 do aIm_crow[i, 2].Visible := aIm_crow[i - 1, 2].Visible; aIm_crow[1, 2].Visible := false;
             { row 4 } aIm_crow[5, 5].Visible := aIm_crow[4, 4].Visible; for i := 4 downto 2 do aIm_crow[i, 4].Visible := aIm_crow[i - 1, 4].Visible; aIm_crow[1, 4].Visible := false;

             // crows counter
             iCrowCounter := 0;

                                for i := 4 to 5 do if aIm_crow[i, 0].Visible then Inc(iCrowCounter);
             for j := 1 to 3 do for i := 1 to 5 do if aIm_crow[i, j].Visible then Inc(iCrowCounter);
                                for i := 1 to 4 do if aIm_crow[i, 4].Visible then Inc(iCrowCounter);
                                for i := 1 to 5 do if aIm_crow[i, 5].Visible then Inc(iCrowCounter);

             // crows generation
             bGenerateCrow := false;

             if Random(iMaxRandom) < 2 then bGenerateCrow := true;
             if iCrowCounter = 0       then bGenerateCrow := true;
             if iCrowCounter >= 6      then bGenerateCrow := false;
             if aIm_crow[5, 0].Visible then bGenerateCrow := false;

             aIm_crow[4, 0].Visible := bGenerateCrow;

             // audio
             if aIm_crow[4, 0].Visible or aIm_crow[5, 0].Visible or aIm_crow[5, 1].Visible then bPlaySound := true;
             if aIm_crow[5, 3].Visible then bPlaySound := true; for i := 2 to 5 do if aIm_crow[i, 2].Visible then bPlaySound := true;
             if aIm_crow[5, 5].Visible then bPlaySound := true; for i := 2 to 4 do if aIm_crow[i, 4].Visible then bPlaySound := true;
           end;
        2: begin                                                                // crows -> even row
             { row 1 } aIm_crow[1, 2].Visible := aIm_crow[1, 1].Visible; for i := 1 to 4 do aIm_crow[i, 1].Visible := aIm_crow[i + 1, 1].Visible; aIm_crow[5, 1].Visible := false;
             { row 3 } aIm_crow[1, 4].Visible := aIm_crow[1, 3].Visible; for i := 1 to 4 do aIm_crow[i, 3].Visible := aIm_crow[i + 1, 3].Visible; aIm_crow[5, 3].Visible := false;
             { row 5 } if aIm_crow[1, 5].Visible then bPlaySound := true;
             { row 5 } for i := 1 to 4 do aIm_crow[i, 5].Visible := aIm_crow[i + 1, 5].Visible; aIm_crow[5, 5].Visible := false;

             // audio
             if aIm_crow[1, 2].Visible then bPlaySound := true; for i := 1 to 4 do if aIm_crow[i, 1].Visible then bPlaySound := true;
             if aIm_crow[1, 4].Visible then bPlaySound := true; for i := 1 to 4 do if aIm_crow[i, 3].Visible then bPlaySound := true;
                                                                for i := 1 to 4 do if aIm_crow[i, 5].Visible then bPlaySound := true;
           end;
      end; // case..of

      if iPosHeroX > 4 then
        bPlaySound := false;

      if bPlaySound then
        Snd_Play(pfs_crow);

      // loops action
      Inc(iLoopGame);

      if iLoopGame = 3 then
        iLoopGame := 1;
    end;
end;





procedure TForm1.Timer_Game_OverTimer(Sender: TObject);
{ the game is over; sets mode to "time", 5 minutes after game is over          }
begin
  // if the last miss has been made in water
  if (iPosHeroX > 4) and (iPosHeroY = 5) then
    aIm_miss[1].Visible := not aIm_miss[1].Visible
  else
    // if the last miss has been made because of a crow
    begin
      aIm_junior[iPosHeroX, iPosHeroY].Visible := not aIm_junior[iPosHeroX, iPosHeroY].Visible;

      // if the last miss has been made in (1,1) -> display Junior's hand too
      if (iPosHeroX = 1) and (iPosHeroY = 1) then
        begin
          aIm_junior_hand_1[2].Visible := false;
          aIm_junior_hand_1[1].Visible := aIm_junior[iPosHeroX, iPosHeroY].Visible;
        end;
    end;

  Inc(iLoopGameOver);

  if iLoopGameOver = 300 then
    begin
      iMode := 4; Gam_Set_Mode;

      Timer_Game_Over.Enabled := false;
    end;
end;





procedure TForm1.Timer_Game_StartTimer(Sender: TObject);
{ the starting tune plays and the game is set                                  }
begin
  case iLoopGameStart of
    1: begin
         Snd_Play(pfs_start1);

         Timer_Game_Start.Interval := 4000;
       end;
    2: begin
         // iLoopKongArm := 1 -> was declared in Gam_Set_Mode with iMode = 6 or 8
         Timer_Kong_Arm.Interval := 740;
         Timer_Kong_Arm.Enabled := true;

         iLoopUmbrellaBalloon := 1;
         Timer_Umbrella_Balloon.Interval := 800;
         Timer_Umbrella_Balloon.Enabled := true;

         bControls := true;

         iLoopGame := 1;
         Gam_Set_Difficulty;
         Timer_Game.Enabled := true;

         Timer_Game_Start.Enabled := false;
       end;
  end; // case..of

  Inc(iLoopGameStart);
end;





procedure TForm1.Timer_Junior_1_1_HandTimer(Sender: TObject);
{ Junior waves his hand when in (1,1)                                          }
begin
  case iLoopJunior11Hand of
    1: begin
         aIm_junior_hand_1[2].Visible := false;

         if aIm_junior[1, 1].Visible then
           aIm_junior_hand_1[1].Visible := true;

           Timer_Junior_1_1_Hand.Interval := 800;
       end;
    2: begin
         bKeypressed := false;

         Timer_Junior_1_1_Hand.Enabled := false;
       end;
  end; // case..of

  Inc(iLoopJunior11Hand);
end;





procedure TForm1.Timer_Junior_6_HandTimer(Sender: TObject);
{ Junior puts his hand to the lock when in (6,x)                               }
begin
  aIm_junior_hand_6[iPosHeroY].Visible := false;

  Snd_Silence;
  Snd_Play(pfs_flying);

  Timer_Flying_Music.Interval := 4060;
  Timer_Flying_Music.Enabled := false;
  Timer_Flying_Music.Enabled := true;

  Timer_Junior_6_Hand.Enabled := false;
end;






procedure TForm1.Timer_Junior_JumpTimer(Sender: TObject);
{ occurs when Junior is in (1,1) only!                                         }
begin
  case iLoopJuniorJump of
    1: begin
         if iKeyStatus = 1 then
           if not aIm_crow[2, 5].Visible then
             Timer_Junior_Jump.Interval := 620
           else
         else
           if not aIm_crow[2, 5].Visible then
             Timer_Junior_Jump.Interval := 560;

         if aIm_crow[1, 4].Visible then
           begin
             Timer_Key_Blink.Enabled := false;
             aIm_key[1].Visible := false;

             Timer_Junior_Jump.Interval := Trunc(Timer_Game.Interval / 2);
           end;

         if aIm_crow[2, 5].Visible then
           begin
             Timer_Key_Blink.Enabled := false;
             aIm_key[1].Visible := false;

             Timer_Junior_Jump.Interval := Timer_Game.Interval * 2;
           end;

         Inc(iLoopJuniorJump);
       end;
    2: begin
         Timer_Key_Blink.Enabled := false;
         aIm_key[1].Visible := false;

         if iKeyStatus = 1 then
           aIm_coconut[0].Visible := true;

         iKeyStatus := 2;

         Vid_Show_Hero(false);
         Inc(iPosHeroY);
         Vid_Show_Hero(true);

         Snd_Play(pfs_step);

         bControls := true;

         Timer_Junior_Jump.Enabled := false;
       end;
  end; // case..of
end;





procedure TForm1.Timer_Junior_StepTimer(Sender: TObject);
{ controller is enabled again                                                  }
begin
  if bKeyPressed then
    bKeypressed := false;

  Timer_Junior_Step.Enabled := false;
end;





procedure TForm1.Timer_Key_BlinkTimer(Sender: TObject);
{ key blinks when in [1,1]                                                     }
begin
  aIm_key[1].Visible := not aIm_key[1].Visible;
end;





procedure TForm1.Timer_Key_FallTimer(Sender: TObject);
{ the key falls into the water                                                 }
begin
  case iLoopKeyFall of
    1: begin
         aIm_junior_hand_6[iPosHeroY].Visible := false;

         Timer_Key_Fall.Interval := 340;
       end;
    2: begin
         aIm_key[2].Visible := true;
         Timer_Key_Fall.Interval := 160;
       end;
    3: aIm_key[2].Visible := false;
    4: aIm_key[2].Visible := true;
    5: begin
         aIm_key[2].Visible := false;
         aIm_miss[2].Visible := true;
       end;
    7: aIm_miss[2].Visible := false;
    9: begin
         iKeyStatus := 1;
         aIm_key[1].Visible := true;
         Timer_Key_Blink.Enabled := true;

         Gam_Set_Difficulty;
         Timer_Game.Enabled := true;

         Timer_Kong_Arm.Interval := 740;
         Timer_Kong_Arm.Enabled := true;

         Timer_Umbrella_Balloon.Interval := 800;
         Timer_Umbrella_Balloon.Enabled := true;

         Snd_Play(pfs_flying);
         Timer_Flying_Music.Interval := 4060;
         Timer_Flying_Music.Enabled := true;

         bControls := true;

         Timer_Key_Fall.Enabled := false;
       end;
  end; // case..of

  Inc(iLoopKeyFall);
end;





procedure TForm1.Timer_Kong_ArmTimer(Sender: TObject);
{ Donkey Kong arms oscillates up-down                                          }
begin
  Vid_Show_Kong_Arm_Lock(false);

  Inc(iLoopKongArm);

  if iLoopKongArm > 6
    then iLoopKongArm := 3;

  Vid_Show_Kong_Arm_Lock(true);
end;





procedure TForm1.Timer_Miss_CrowTimer(Sender: TObject);
{ Junior has hit a crow                                                        }
begin
  case iLoopMissCrow of
    1: Timer_Miss_Crow.Interval := 200;
    2: begin
         Vid_Show_Hero(false);

         if (iPosHeroX = 1) and (iPosHeroY = 1) then
           begin
             aIm_junior_hand_1[1].Visible := false;
             aIm_junior_hand_1[2].Visible := false;
           end;

         Timer_Miss_Crow.Interval := 450;
       end;
    3, 5: begin Vid_Show_Hero(true);  if (iPosHeroX = 1) and (iPosHeroY = 1) then aIm_junior_hand_1[1].Visible := true; Timer_Miss_Crow.Interval := 250; end;
    4: begin Vid_Show_Hero(false); if (iPosHeroX = 1) and (iPosHeroY = 1) then aIm_junior_hand_1[1].Visible := false; Timer_Miss_Crow.Interval := 450; end;
    6: begin Vid_Show_Hero(false); if (iPosHeroX = 1) and (iPosHeroY = 1) then aIm_junior_hand_1[1].Visible := false; Timer_Miss_Crow.Interval := 800; end;
    7: begin
         Gam_Reprise_After_Miss;

         Timer_Miss_Crow.Enabled := false;
       end;
  end; // case..of

  Inc(iLoopMissCrow);
end;





procedure TForm1.Timer_Miss_FallTimer(Sender: TObject);
{ Junior is falling into water                                                 }
{ the procedure is called with "iLoopMissFall" equaled to 1, 2, 3 or 4,        }
{ depending on the "Y" coordinate of Junior                                    }
begin
  case iLoopMissFall of
    1, 2:     begin Vid_Show_Hero(false); Inc(iPosHeroY); Vid_Show_Hero(true);           Timer_Miss_Fall.Interval := 180; end;
    3:        begin Vid_Show_Hero(false); Inc(iPosHeroY); Vid_Show_Hero(true);           Timer_Miss_Fall.Interval := 300; end;
    4:        begin Vid_Show_Hero(false); Inc(iPosHeroY); aIm_miss[1].Visible := true;   Timer_Miss_Fall.Interval := 1;   end;
    5:        begin Snd_Silence; Snd_Play(pfs_miss);                                     Timer_Miss_Fall.Interval := 840; end;
    6, 8, 10: begin aIm_miss[2].Visible := true;  Timer_Miss_Fall.Interval := 160; end;
    7, 9:     begin aIm_miss[2].Visible := false; Timer_Miss_Fall.Interval := 310; end;
   11:        begin aIm_miss[2].Visible := false; Timer_Miss_Fall.Interval := 340; end;
   12, 14:    begin aIm_miss[1].Visible := false; end;
   13, 15:    begin aIm_miss[1].Visible := true;  end;
   16:        begin aIm_miss[1].Visible := false; Timer_Miss_Fall.Interval := 800; end;
   17: begin
         Gam_Reprise_After_Miss;

         Timer_Miss_Fall.Enabled := false;
       end;
  end; // case..of

  Inc(iLoopMissFall);
end;





procedure TForm1.Timer_Score_BlinkTimer(Sender: TObject);
{ score blinks during chance time                                              }
begin
  Vid_Show_Panel_Score(iScore, not Odd(iLoopScoreBlink));

  Inc(iLoopScoreBlink);

  if iLoopScoreBlink >= 3 then
    iLoopScoreBlink := 1;
end;





procedure TForm1.Timer_Umbrella_BalloonTimer(Sender: TObject);
{ umbrellas and balloons move                                                  }
  var
    i,
    iOnScreen: integer;

    bGenerate: boolean;
begin
  case iLoopUmbrellaBalloon of
    1: begin                                                                    // umbrellas turn
         // counts umbrellas on screen
         iOnScreen := 0;

         for i := 1 to 5 do
           if aIm_umbrella[i].Visible then
             Inc(iOnScreen);

          // moves existing umbrellas
          for i := 5 downto 2 do
            aIm_umbrella[i].Visible := aIm_umbrella[i - 1].Visible;

          // generates/doesn't generate new umbrella
          bGenerate := false;

          if   (Random(2) = 0)
            or (iOnScreen = 0)
            or (aIm_junior[6, 1].Visible) then
            bGenerate := true;

         if aIm_umbrella[2].Visible then
           bGenerate := false;

         aIm_umbrella[1].Visible := bGenerate;

         // Junior movement if hanging from an umbrella
         if iPosHeroX = 5 then
           begin
             Vid_Show_Hero(false);
             Inc(iPosHeroY);

             if iPosHeroY < 5 then
               Vid_Show_Hero(true)
             else
               begin
                 iPosHeroY := 4;
                 Gam_Execute_Miss_Fall;
               end;
           end;

         iLoopUmbrellaBalloon := 2;
       end;
    2: begin                                                                    // balloons turn
         // counts balloons on screen
         iOnScreen := 0;

         for i := 0 to 5 do
           if aIm_balloon[i].Visible then
             Inc(iOnScreen);

          // moves existing balloons
         for i := 0 to 4 do
           aIm_balloon[i].Visible := aIm_balloon[i + 1].Visible;

         // generates/doesn't generate new balloon
         bGenerate := false;

         if (Random(2) = 0)
           or (iOnScreen = 0)
           or (aIm_junior[5, 3].Visible) then
           bGenerate := true;

         if aIm_balloon[4].Visible then
           bGenerate := false;

         aIm_balloon[5].Visible := bGenerate;

         // Junior movement if hanging from a balloon
         if iPosHeroX = 6 then
           begin
             Dec(iPosHeroY);
             aIm_junior[iPosHeroX, iPosHeroY + 1].Visible := false;

             for i := 2 to 4 do
               aIm_junior_hand_6[i].Visible := false;

             if iPosHeroY > 0 then
               Vid_Show_Hero(true)
             else
               begin
                 iPosHeroY := 1;
                 Gam_Execute_Miss_Fall;
               end;
           end;

         iLoopUmbrellaBalloon := 1;
       end;
  end; // case..of
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: POPUP MENU MANAGEMENT                                        * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.pm_0_menuPopup(Sender: TObject);
{ shows the "checked" menuitems if needed. also, hides the game menu if it is  }
{ visible when the popup menu pops up                                          }
begin
{}pm_11_splashscreen.Checked := iReg.ReadBool('', 'SplashScreen', SplashForm.chb_splashscreen.Checked);
{}pm_12_controller.Checked := bDisplayCtrlOverlays;

{}if bFullscreenMode
    and bGameMenuOpen then
    begin
      bGameMenuOpen := false;
      Vid_Display_Game_Menu(bGameMenuOpen);
    end;
end;





procedure TForm1.pm_1_aboutClick(Sender: TObject);
{ displays the embedded "About the author..." form                             }
begin
  AboutForm.Show;
end;





procedure TForm1.pm_2_readmeClick(Sender: TObject);
{ opens the "Readme.txt" file                                                  }
begin
  ShellExecute(Handle, 'open', 'Readme.txt', nil, '', SW_SHOWNORMAL);
end;





procedure TForm1.pm_3_whatsnewClick(Sender: TObject);
{ opens the "Whatsnew.txt" file                                                }
begin
  ShellExecute(Handle, 'open', 'Whatsnew.txt', nil, '', SW_SHOWNORMAL);
end;





procedure TForm1.pm_4_manualClick(Sender: TObject);
{ displays the embedded instructions manual                                    }
begin
  ManualForm.Show;
end;





procedure TForm1.pm_6_configClick(Sender: TObject);
{ opens the "Config.ini" file                                                  }
begin
  ShellExecute(Handle, 'open', 'Config.ini', nil, '', SW_SHOWNORMAL);
end;





procedure TForm1.pm_7_keysClick(Sender: TObject);
{ displays the "key settings" form                                             }
begin
  KeysForm.position := poScreenCenter;
  KeysForm.visible := true;
  Enabled := false;
end;





procedure TForm1.pm_8_hiscoreClick(Sender: TObject);
{ resets high scores to the system registry                                    }
begin
  iHiScoreA := 0; iReg.WriteInteger('', 'HiScore1', iHiScoreA);
  iHiScoreB := 0; iReg.WriteInteger('', 'HiScore2', iHiScoreB);
  Beep;
end;





{}procedure TForm1.pm_10_fullscreenClick(Sender: TObject);
{ changes from fullscreen to windowed, and vice-versa                          }
begin
  // updates popup menu
  bFullscreenMode := not bFullscreenMode;
  pm_10_fullscreen.Checked := bFullscreenMode;
  bGameMenuOpen := false;

  iReg.WriteBool('', 'FullscreenMode', bFullscreenMode);

  // graphics init
  Top  := - Screen.Height;
  Left := - Screen.Width;
  Gfx_Game_Init;
  Position := poScreenCenter;
end;





procedure TForm1.pm_11_splashscreenClick(Sender: TObject);
{ updates the ticks to the checkbox, the popup menu and the system registry    }
begin
{}pm_11_splashscreen.Checked := not pm_11_splashscreen.Checked;

{}iReg.WriteBool('', 'SplashScreen', pm_11_splashscreen.Checked);
end;





procedure TForm1.pm_12_controllerClick(Sender: TObject);
{ displays the "tranlucent" controller overlay on the directional cross        }
begin
{}pm_12_controller.Checked := not pm_12_controller.Checked;
{}bDisplayCtrlOverlays := pm_12_controller.Checked;

{}iReg.WriteBool('', 'CtrlrOverlay', pm_12_controller.Checked);

  Vid_Display_Ctrl_Overlay;
end;





procedure TForm1.pm_14_exitClick(Sender: TObject);
{ exits the game                                                               }
begin
  Application.Terminate;
end;










{ **************************************************************************** }
{ *                                                                          * }
{ * PROCEDURES: GAME MENU MANAGEMENT                                         * }
{ *                                                                          * }
{ **************************************************************************** }

procedure TForm1.btn_game_menu_topClick(Sender: TObject);
{ sets the game menu as visible/hidden, then displays/hides it                 }
begin
  bGameMenuOpen := not bGameMenuOpen;
  Vid_Display_Game_Menu(bGameMenuOpen);
end;





procedure TForm1.la_menu_itemMouseEnter(Sender: TObject);
{ the game menu label bg colour becomes solid                                  }
begin
  if Sender is TLabel then
    TLabel(Sender).Transparent := false;
end;





procedure TForm1.la_menu_itemMouseLeave(Sender: TObject);
{ the game menu label bg colour becomes transparent                            }
begin
  if Sender is TLabel then
    TLabel(Sender).Transparent := true;
end;





procedure TForm1.la_menu_itemMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
{ hides the game menu, once a label has been pressed                           }
begin
  if Button = mbLeft then
    begin
      bGameMenuOpen := false;
      Vid_Display_Game_Menu(false);
    end;
end;





procedure TForm1.Vid_Display_Game_Menu(b: boolean);
{ displays/hides the game menu, depending on the value of bGameMenuOpen        }
  var
    i: integer;
begin
  case bGameMenuOpen of
    true: begin
        for i := 1 to MENU_ITEMS do
          begin
            aMenuLabels[i] := TLabel.Create(Form1);
            aMenuLabels[i].Parent := Form1;

            aMenuLabels[i].Top := 32 * i;
            aMenuLabels[i].Left := 0;

            aMenuLabels[i].AutoSize := false;
            aMenuLabels[i].Height := 32;
            aMenuLabels[i].Width := 256;
            aMenuLabels[i].Layout := tlCenter;

            aMenuLabels[i].Transparent := true;

            if aMenuItem[i].Caption <> '-' then
              begin
                aMenuLabels[i].Font.Name := 'Arial';
                aMenuLabels[i].Font.Color := clWhite;
                aMenuLabels[i].Font.Size := -12;
                aMenuLabels[i].Caption := '   ' + aMenuItem[i].Caption;

                if aMenuItem[i].Checked then
                  aMenuLabels[i].Caption := aMenuLabels[i].Caption + ' [V]';

                aMenuLabels[i].Color := $00000000;
                aMenuLabels[i].Cursor := crHandPoint;
                aMenuLabels[i].Enabled := aMenuItem[i].Enabled;

                if aMenuLabels[i].Enabled then
                  begin
                    aMenuLabels[i].OnClick      := aMenuItem[i].OnClick;
                    aMenuLabels[i].OnMouseEnter := la_menu_itemMouseEnter;
                    aMenuLabels[i].OnMouseLeave := la_menu_itemMouseLeave;
                    aMenuLabels[i].OnMouseUp    := la_menu_itemMouseUp;
                  end;
              end;
          end;

        btn_game_menu_background.Width := 256;
        btn_game_menu_background.Height := 32 * (MENU_ITEMS + 1)
      end;
    false: begin
        for i := 1 to MENU_ITEMS do
          begin
            aMenuLabels[i].Free;
          end;

        btn_game_menu_background.Height := 32;
        btn_game_menu_background.Width := 36;
      end;
    end; // case..of
end;





initialization
  bStartup := true;





end.
